@@ -1,12 +1,9 @@
 /*
  * config.h
  *
- * Copyright (c) Tuomo Valkonen 1999-2002. 
+ * Copyright (c) Tuomo Valkonen 1999-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * FOOBARBAZQUK
  */
 
 #ifndef ION_CONFIG_H
@@ -21,7 +18,6 @@
 
 #define CF_MAX_MOVERES_STR_SIZE 32
 
-#define CF_TAB_TEXT_ALIGN ALIGN_CENTER
 #define CF_RESIZE_DELAY 1500
 
 #define CF_XMESSAGE "xmessage -file "
@@ -41,9 +37,6 @@
 #define CF_CURSOR_DRAG XC_cross
 #define CF_CURSOR_WAITKEY XC_icon
 
-#define CF_STR_EMPTY "<empty frame>"
-#define CF_STR_EMPTY_LEN 13
-
 #define CF_STDISP_MIN_SZ 8
 
 #endif /* ION_CONFIG_H */
