@@ -1,12 +1,9 @@
 /*
  * ion/de/colour.h
  *
- * Copyright (c) Tuomo Valkonen 1999-2004. 
+ * Copyright (c) Tuomo Valkonen 1999-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #include <ioncore/common.h>
@@ -21,8 +18,8 @@ bool de_alloc_colour(WRootWin *rootwin, DEColour *ret, const char *name)
     if(name==NULL)
         return FALSE;
 
-    if(XParseColor(wglobal.dpy, rootwin->default_cmap, name, &c)){
-        ok=XAllocColor(wglobal.dpy, rootwin->default_cmap, &c);
+    if(XParseColor(ioncore_g.dpy, rootwin->default_cmap, name, &c)){
+        ok=XAllocColor(ioncore_g.dpy, rootwin->default_cmap, &c);
         if(ok)
             *ret=c.pixel;
     }
@@ -35,8 +32,8 @@ bool de_duplicate_colour(WRootWin *rootwin, DEColour in, DEColour *out)
 {
     XColor c;
     c.pixel=in;
-    XQueryColor(wglobal.dpy, rootwin->default_cmap, &c);
-    if(XAllocColor(wglobal.dpy, rootwin->default_cmap, &c)){
+    XQueryColor(ioncore_g.dpy, rootwin->default_cmap, &c);
+    if(XAllocColor(ioncore_g.dpy, rootwin->default_cmap, &c)){
         *out=c.pixel;
         return TRUE;
     }
@@ -54,12 +51,9 @@ void de_free_colour_group(WRootWin *rootwin, DEColourGroup *cg)
     pixels[3]=cg->sh;
     pixels[4]=cg->pad;
     
-    XFreeColors(wglobal.dpy, rootwin->default_cmap, pixels, 5, 0);
+    XFreeColors(ioncore_g.dpy, rootwin->default_cmap, pixels, 5, 0);
     
-    if(cg->spec!=NULL){
-        free(cg->spec);
-        cg->spec=NULL;
-    }
+    gr_stylespec_unalloc(&cg->spec);
 }
 
 
@@ -69,6 +63,6 @@ void de_free_colour(WRootWin *rootwin, DEColour col)
     
     pixels[0]=col;
     
-    XFreeColors(wglobal.dpy, rootwin->default_cmap, pixels, 1, 0);
+    XFreeColors(ioncore_g.dpy, rootwin->default_cmap, pixels, 1, 0);
 }
 
