@@ -1,12 +1,9 @@
 /*
  * ion/de/colour.h
  *
- * Copyright (c) Tuomo Valkonen 1999-2006. 
+ * Copyright (c) Tuomo Valkonen 1999-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #ifndef ION_DE_COLOUR_H
@@ -24,7 +21,7 @@ typedef unsigned long DEColour;
 
 
 DECLSTRUCT(DEColourGroup){
-    char *spec;
+    GrStyleSpec spec;
     DEColour bg, hl, sh, fg, pad;
 };
 
