@@ -1,12 +1,9 @@
 /*
  * ion/de/font.h
  *
- * Copyright (c) Tuomo Valkonen 1999-2005. 
+ * Copyright (c) Tuomo Valkonen 1999-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #ifndef ION_DE_FONT_H
@@ -38,8 +35,7 @@ extern DEFont *de_load_font(const char *fontname);
 extern void de_free_font(DEFont *font);
 
 extern void debrush_draw_string(DEBrush *brush, int x, int y,
-                                const char *str, int len, bool needfill,
-                                const char *attrib);
+                                const char *str, int len, bool needfill);
 extern void debrush_do_draw_string(DEBrush *brush, int x, int y,
                                    const char *str, int len, bool needfill, 
                                    DEColourGroup *colours);
