@@ -4,32 +4,38 @@
 
 $key = q/-1;MSF=1.6;LFS=11;AAT/;
 $cached_env_img{$key} = q|<IMG
- WIDTH="26" HEIGHT="31" ALIGN="MIDDLE" BORDER="0"
+ WIDTH="27" HEIGHT="31" ALIGN="MIDDLE" BORDER="0"
  SRC="|."$dir".q|img4.png"
  ALT="$-1$">|; 
 
 $key = q/(0,1];MSF=1.6;LFS=11;AAT/;
 $cached_env_img{$key} = q|<IMG
- WIDTH="40" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
+ WIDTH="42" HEIGHT="36" ALIGN="MIDDLE" BORDER="0"
  SRC="|."$dir".q|img3.png"
  ALT="$(0, 1]$">|; 
 
 $key = q/N=1{ldots}5;MSF=1.6;LFS=11;AAT/;
 $cached_env_img{$key} = q|<IMG
- WIDTH="80" HEIGHT="15" ALIGN="BOTTOM" BORDER="0"
+ WIDTH="82" HEIGHT="15" ALIGN="BOTTOM" BORDER="0"
  SRC="|."$dir".q|img2.png"
  ALT="$N=1{\ldots} 5$">|; 
 
+$key = q/-2;MSF=1.6;LFS=11;AAT/;
+$cached_env_img{$key} = q|<IMG
+ WIDTH="27" HEIGHT="31" ALIGN="MIDDLE" BORDER="0"
+ SRC="|."$dir".q|img5.png"
+ ALT="$-2$">|; 
+
 $key = q/n;MSF=1.6;LFS=11;AAT/;
 $cached_env_img{$key} = q|<IMG
- WIDTH="14" HEIGHT="14" ALIGN="BOTTOM" BORDER="0"
+ WIDTH="15" HEIGHT="14" ALIGN="BOTTOM" BORDER="0"
  SRC="|."$dir".q|img1.png"
  ALT="$n$">|; 
 
 $key = q/{t,m,b}times{t,c,b};MSF=1.6;LFS=11;AAT/;
 $cached_env_img{$key} = q|<IMG
- WIDTH="141" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
- SRC="|."$dir".q|img5.png"
+ WIDTH="142" HEIGHT="36" ALIGN="MIDDLE" BORDER="0"
+ SRC="|."$dir".q|img6.png"
  ALT="$\{t,m,b\}\times\{t,c,b\}$">|; 
 
 1;
