@@ -1,12 +1,9 @@
 /*
  * ion/ioncore/attach.h
  *
- * Copyright (c) Tuomo Valkonen 1999-2006. 
+ * Copyright (c) Tuomo Valkonen 1999-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #ifndef ION_IONCORE_ATTACH_H
@@ -54,6 +51,9 @@ extern WRegion *region_attach_helper(WRegion *mgr,
                                      WRegionDoAttachFn *fn, void *fn_param,
                                      const WRegionAttachData *data);
 
-extern bool region_attach_reparent_check(WRegion *mgr, WRegion *reg);
+extern bool region_ancestor_check(WRegion *dst, WRegion *reg);
+
+extern void region_postdetach_dispose(WRegion *reg, WRegion *disposeroot);
+
 
 #endif /* ION_IONCORE_ATTACH_H */
