@@ -1,12 +1,9 @@
 /*
  * ion/ioncore/basicpholder.h
  *
- * Copyright (c) Tuomo Valkonen 2005-2006. 
+ * Copyright (c) Tuomo Valkonen 2005-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #ifndef ION_IONCORE_BASICPHOLDER_H
@@ -17,8 +14,8 @@
 #include "attach.h"
 
 
-typedef bool WBasicPHolderHandler(WRegion *reg, int flags,
-                                  WRegionAttachData *data);
+typedef WRegion *WBasicPHolderHandler(WRegion *reg, int flags,
+                                      WRegionAttachData *data);
 
 INTRCLASS(WBasicPHolder);
 
@@ -40,7 +37,7 @@ extern bool basicpholder_do_goto(WBasicPHolder *ph);
 
 extern WRegion *basicpholder_do_target(WBasicPHolder *ph);
 
-extern bool basicpholder_do_attach(WBasicPHolder *ph, int flags,
-                                   WRegionAttachData *data);
+extern WRegion *basicpholder_do_attach(WBasicPHolder *ph, int flags,
+                                       WRegionAttachData *data);
 
 #endif /* ION_IONCORE_BASICPHOLDER_H */
