@@ -1,12 +1,9 @@
 /*
  * ion/ioncore/binding.c
  *
- * Copyright (c) Tuomo Valkonen 1999-2006. 
+ * Copyright (c) Tuomo Valkonen 1999-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #include <string.h>
@@ -486,6 +483,11 @@ static WBinding *do_bindmap_lookup_binding(WBindmap *bindmap,
     
     binding=search_binding(bindmap, &tmp);
 
+    if(BINDING_IS_PSEUDO(act)){
+        /* No use trying anything else */
+        return binding;
+    }
+    
     if(binding==NULL){
         tmp.state=AnyModifier;
         binding=search_binding(bindmap, &tmp);
