@@ -1,12 +1,9 @@
 /*
  * ion/ioncore/classes.h
  *
- * Copyright (c) Tuomo Valkonen 1999-2006. 
+ * Copyright (c) Tuomo Valkonen 1999-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #ifndef ION_IONCORE_CLASSES_H
@@ -33,6 +30,7 @@ INTRCLASS(WGroupWS);
 
 INTRCLASS(WPHolder);
 INTRCLASS(WMPlexPHolder);
+INTRCLASS(WFramedPHolder);
 
 INTRSTRUCT(WStacking);
 INTRSTRUCT(WLListNode);
@@ -42,4 +40,6 @@ INTRSTRUCT(WSubmapState);
 
 INTRSTRUCT(WRegionAttachData);
 
+INTRSTRUCT(WRQGeomParams);
+
 #endif /* ION_IONCORE_CLASSES_H */
