@@ -1,12 +1,9 @@
 /*
  * ion/ioncore/event.c
  *
- * Copyright (c) Tuomo Valkonen 1999-2006. 
+ * Copyright (c) Tuomo Valkonen 1999-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #include <X11/Xmd.h>
@@ -160,38 +157,41 @@ void ioncore_get_event(XEvent *ev, long mask)
 /*{{{ Flush */
 
 
-static void skip_focusenter()
+static void skip_enterwindow()
 {
     XEvent ev;
-    WRegion *r;
     
     XSync(ioncore_g.dpy, False);
     
-    while(XCheckMaskEvent(ioncore_g.dpy,
-                          EnterWindowMask|FocusChangeMask, &ev)){
+    while(XCheckMaskEvent(ioncore_g.dpy, EnterWindowMask, &ev)){
         ioncore_update_timestamp(&ev);
-        if(ev.type==FocusOut)
-            ioncore_handle_focus_out(&(ev.xfocus));
-        else if(ev.type==FocusIn)
-            ioncore_handle_focus_in(&(ev.xfocus), TRUE);
-        /*else if(ev.type==EnterNotify)
-            handle_enter_window(&ev);*/
     }
 }
 
 
-void ioncore_flush()
+void ioncore_flushfocus()
 {
-    if(ioncore_g.focus_next!=NULL && 
-       ioncore_g.input_mode==IONCORE_INPUTMODE_NORMAL){
-        bool warp=ioncore_g.warp_next;
-        WRegion *next=ioncore_g.focus_next;
-        ioncore_g.focus_next=NULL;
-        skip_focusenter();
-        region_do_set_focus(next, warp);
-    }
+    WRegion *next;
+    bool warp;
     
-    XFlush(ioncore_g.dpy);
+    if(ioncore_g.input_mode!=IONCORE_INPUTMODE_NORMAL)
+        return;
+        
+    next=ioncore_g.focus_next;
+    warp=ioncore_g.warp_next;
+
+    if(next==NULL)
+        return;
+        
+    ioncore_g.focus_next=NULL;
+        
+    region_do_set_focus(next, warp);
+        
+    /* Just greedily eating it all away that X has to offer
+     * seems to be the best we can do with Xlib.
+     */
+    if(warp)
+        skip_enterwindow();
 }
 
 
@@ -227,12 +227,22 @@ void ioncore_mainloop()
     while(1){
         check_signals();
         mainloop_execute_deferred();
-        ioncore_flush();
-
-        if(QLength(ioncore_g.dpy)>0)
-           ioncore_x_connection_handler(ioncore_g.conn, NULL);
-        else
-            mainloop_select();
+        
+        if(QLength(ioncore_g.dpy)==0){
+            XSync(ioncore_g.dpy, False);
+            
+            if(QLength(ioncore_g.dpy)==0){
+                ioncore_flushfocus();
+                XSync(ioncore_g.dpy, False);
+                
+                if(QLength(ioncore_g.dpy)==0){
+                    mainloop_select();
+                    continue;
+                }
+            }
+        }
+        
+        ioncore_x_connection_handler(ioncore_g.conn, NULL);
     }
 }
 
