@@ -1,12 +1,9 @@
 /*
  * ion/ioncore/float-placement.h
  *
- * Copyright (c) Tuomo Valkonen 1999-2006. 
+ * Copyright (c) Tuomo Valkonen 1999-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #ifndef ION_IONCORE_FLOAT_PLACEMENT_H
@@ -22,6 +19,7 @@ typedef enum{
 
 extern WFloatPlacement ioncore_placement_method;
 
-extern void group_calc_placement(WGroup *ws, WRectangle *geom);
+extern void group_calc_placement(WGroup *ws, uint level, 
+                                 WRectangle *geom);
 
 #endif /* ION_IONCORE_FLOAT_PLACEMENT_H */
