@@ -1,12 +1,9 @@
 /*
  * ion/ioncore/frame-draw.h
  *
- * Copyright (c) Tuomo Valkonen 1999-2006. 
+ * Copyright (c) Tuomo Valkonen 1999-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #ifndef ION_IONCORE_FRAME_DRAW_H
@@ -35,4 +32,12 @@ extern void frame_clear_shape(WFrame *frame);
 extern const char *framemode_get_style(WFrameMode mode);
 extern const char *framemode_get_tab_style(WFrameMode mode);
 
+extern void frame_update_attr(WFrame *frame, int i, WRegion *reg);
+
+extern void frame_setup_dragwin_style(WFrame *frame, GrStyleSpec *spec, int tab);
+
+extern void frame_inactivated(WFrame *frame);
+extern void frame_activated(WFrame *frame);
+extern void frame_quasiactivity_change(WFrame *frame);
+
 #endif /* ION_IONCORE_FRAME_DRAW_H */
