@@ -1,12 +1,9 @@
 /*
  * ion/ioncore/frame-pointer.c
  *
- * Copyright (c) Tuomo Valkonen 1999-2006. 
+ * Copyright (c) Tuomo Valkonen 1999-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #include <string.h>
@@ -138,7 +135,7 @@ static bool tabdrag_kbd_handler(WRegion *reg, XEvent *xev)
     
     assert(reg!=NULL);
 
-    binding=bindmap_lookup_binding(ioncore_rootwin_bindmap, BINDING_KEYPRESS,
+    binding=bindmap_lookup_binding(ioncore_screen_bindmap, BINDING_KEYPRESS,
                                    ev->state&~BUTTONS_MASK, ev->keycode);
     
     if(binding!=NULL && binding->func!=extl_fn_none()){
@@ -173,9 +170,7 @@ static void setup_dragwin(WFrame *frame, uint tab)
     if(tabdrag_infowin==NULL)
         return;
     
-    infowin_set_attr2(tabdrag_infowin, (REGION_IS_ACTIVE(frame) 
-                                        ? "active" : "inactive"),
-                      frame->titles[tab].attr);
+    frame_setup_dragwin_style(frame, infowin_stylespec(tabdrag_infowin), tab);
     
     if(frame->titles[tab].text!=NULL){
         char *buf=INFOWIN_BUFFER(tabdrag_infowin);
@@ -338,7 +333,7 @@ static void p_tabdrag_end(WFrame *frame, XButtonEvent *ev)
 /*EXTL_DOC
  * Start dragging the tab that the user pressed on with the pointing device.
  * This function should only be used by binding it to \emph{mpress} or
- * \emph{mdrag} action with area ''tab''.
+ * \emph{mdrag} action with area \codestr{tab}.
  */
 EXTL_EXPORT_MEMBER
 void frame_p_tabdrag(WFrame *frame)
