@@ -1,12 +1,9 @@
 /*
  * ion/ioncore/frame.c
  *
- * Copyright (c) Tuomo Valkonen 1999-2006. 
+ * Copyright (c) Tuomo Valkonen 1999-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #include <string.h>
@@ -37,8 +34,9 @@
 #include "bindmaps.h"
 #include "regbind.h"
 #include "gr.h"
-#include "activity.h"
 #include "llist.h"
+#include "framedpholder.h"
+#include "return.h"
 
 
 extern bool frame_set_background(WFrame *frame, bool set_always);
@@ -83,6 +81,9 @@ bool frame_init(WFrame *frame, WWindow *parent, const WFitParams *fp,
     frame->mode=mode;
     frame->tab_min_w=0;
     frame->bar_max_width_q=1.0;
+    frame->quasiact_source=NULL;
+    
+    gr_stylespec_init(&frame->baseattr);
     
     if(!mplex_init((WMPlex*)frame, parent, fp))
         return FALSE;
@@ -91,7 +92,7 @@ bool frame_init(WFrame *frame, WWindow *parent, const WFitParams *fp,
     frame_initialise_titles(frame);
     
     region_add_bindmap((WRegion*)frame, ioncore_frame_bindmap);
-    region_add_bindmap((WRegion*)frame, ioncore_frame_toplevel_bindmap);
+    region_add_bindmap((WRegion*)frame, ioncore_mplex_bindmap);
     
     frame_add_mode_bindmaps(frame);
     
@@ -116,6 +117,7 @@ void frame_deinit(WFrame *frame)
 {
     frame_free_titles(frame);
     frame_release_brushes(frame);
+    gr_stylespec_unalloc(&frame->baseattr);
     mplex_deinit((WMPlex*)frame);
 }
 
@@ -130,15 +132,19 @@ static void frame_add_mode_bindmaps(WFrame *frame)
 {
     WFrameMode mode=frame->mode;
     
-    if(mode==FRAME_MODE_TILED || mode==FRAME_MODE_TILED_ALT){
-	region_add_bindmap((WRegion*)frame, ioncore_frame_toplevel_bindmap);
-        region_add_bindmap((WRegion*)frame, ioncore_frame_tiled_bindmap);
-    }else if(mode==FRAME_MODE_FLOATING){
+    if(mode==FRAME_MODE_FLOATING){
+	region_add_bindmap((WRegion*)frame, ioncore_mplex_toplevel_bindmap);
 	region_add_bindmap((WRegion*)frame, ioncore_frame_toplevel_bindmap);
         region_add_bindmap((WRegion*)frame, ioncore_frame_floating_bindmap);
     }else if(mode==FRAME_MODE_TRANSIENT){
         region_add_bindmap((WRegion*)frame, ioncore_frame_transient_bindmap);
-    }
+        region_add_bindmap((WRegion*)frame, ioncore_frame_floating_bindmap);
+    }else{
+        /* mode==FRAME_MODE_TILED || mode==FRAME_MODE_TILED_ALT || mode==FRAME_MODE_UNKNOWN */
+	region_add_bindmap((WRegion*)frame, ioncore_mplex_toplevel_bindmap);
+	region_add_bindmap((WRegion*)frame, ioncore_frame_toplevel_bindmap);
+        region_add_bindmap((WRegion*)frame, ioncore_frame_tiled_bindmap);
+    } 
 }
 
 
@@ -151,6 +157,7 @@ void frame_set_mode(WFrame *frame, WFrameMode mode)
     
     frame_release_brushes(frame);
     
+    region_remove_bindmap((WRegion*)frame, ioncore_mplex_toplevel_bindmap);
     region_remove_bindmap((WRegion*)frame, ioncore_frame_toplevel_bindmap);
     region_remove_bindmap((WRegion*)frame, ioncore_frame_tiled_bindmap);
     region_remove_bindmap((WRegion*)frame, ioncore_frame_floating_bindmap);
@@ -175,6 +182,7 @@ WFrameMode frame_mode(WFrame *frame)
 
 
 StringIntMap frame_modes[]={
+    {"unknown", FRAME_MODE_UNKNOWN},
     {"tiled", FRAME_MODE_TILED},
     {"tiled-alt", FRAME_MODE_TILED_ALT},
     {"floating", FRAME_MODE_FLOATING},
@@ -186,6 +194,7 @@ StringIntMap frame_modes[]={
 /*EXTL_DOC
  * Get frame mode.
  */
+EXTL_SAFE
 EXTL_EXPORT_AS(WFrame, mode)
 const char *frame_mode_extl(WFrame *frame)
 {
@@ -299,46 +308,6 @@ int frame_nth_tab_iw(WFrame *frame, int n)
 
 
 
-static void update_attr(WFrame *frame, int i, WRegion *reg)
-{
-    int flags=0;
-    static char *attrs[]={
-        "unselected-not_tagged-not_dragged-no_activity",
-        "selected-not_tagged-not_dragged-no_activity",
-        "unselected-tagged-not_dragged-no_activity",
-        "selected-tagged-not_dragged-no_activity",
-        "unselected-not_tagged-dragged-no_activity",
-        "selected-not_tagged-dragged-no_activity",
-        "unselected-tagged-dragged-no_activity",
-        "selected-tagged-dragged-no_activity",
-        "unselected-not_tagged-not_dragged-activity",
-        "selected-not_tagged-not_dragged-activity",
-        "unselected-tagged-not_dragged-activity",
-        "selected-tagged-not_dragged-activity",
-        "unselected-not_tagged-dragged-activity",
-        "selected-not_tagged-dragged-activity",
-        "unselected-tagged-dragged-activity",
-        "selected-tagged-dragged-activity"
-    };
-
-    if(i>=frame->titles_n){
-        /* Might happen when deinitialising */
-        return;
-    }
-    
-    if(reg==FRAME_CURRENT(frame))
-        flags|=0x01;
-    if(reg!=NULL && reg->flags&REGION_TAGGED)
-        flags|=0x02;
-    if(i==frame->tab_dragged_idx)
-        flags|=0x04;
-    if(reg!=NULL && region_is_activity_r(reg))
-        flags|=0x08;
-    
-    frame->titles[i].attr=attrs[flags];
-}
-
-
 void frame_update_attr_nth(WFrame *frame, int i)
 {
     WRegion *reg;
@@ -346,18 +315,18 @@ void frame_update_attr_nth(WFrame *frame, int i)
     if(i<0 || i>=frame->titles_n)
         return;
 
-    update_attr(frame, i, mplex_mx_nth((WMPlex*)frame, i));
+    frame_update_attr(frame, i, mplex_mx_nth((WMPlex*)frame, i));
 }
 
 
-static void update_attrs(WFrame *frame)
+static void frame_update_attrs(WFrame *frame)
 {
     int i=0;
     WRegion *sub;
     WLListIterTmp tmp;
     
     FRAME_MX_FOR_ALL(sub, frame, tmp){
-        update_attr(frame, i, sub);
+        frame_update_attr(frame, i, sub);
         i++;
     }
 }
@@ -371,6 +340,7 @@ static void frame_free_titles(WFrame *frame)
         for(i=0; i<frame->titles_n; i++){
             if(frame->titles[i].text)
                 free(frame->titles[i].text);
+            gr_stylespec_unalloc(&frame->titles[i].attr);
         }
         free(frame->titles);
         frame->titles=NULL;
@@ -383,7 +353,10 @@ static void do_init_title(WFrame *frame, int i, WRegion *sub)
 {
     frame->titles[i].text=NULL;
     frame->titles[i].iw=frame_nth_tab_iw(frame, i);
-    update_attr(frame, i, sub);
+    
+    gr_stylespec_init(&frame->titles[i].attr);
+    
+    frame_update_attr(frame, i, sub);
 }
 
 
@@ -485,6 +458,9 @@ void frame_size_hints(WFrame *frame, WSizeHints *hints_ret)
         if(!USE_MINMAX(frame)){
             hints_ret->max_set=0;
             hints_ret->min_set=0;
+            /*hints_ret->base_set=0;*/
+            hints_ret->aspect_set=0;
+            hints_ret->no_constrain=FALSE;
             /*hints_ret->no_constrain=TRUE;*/
         }
     }else{
@@ -536,15 +512,88 @@ void frame_size_hints(WFrame *frame, WSizeHints *hints_ret)
 /*{{{ Focus  */
 
 
-void frame_inactivated(WFrame *frame)
+static void frame_quasiactivation(WFrame *frame, Obj *src, bool act)
 {
-    window_draw((WWindow*)frame, FALSE);
+    if(frame->quasiact_source==src || act){
+        bool was, is;
+        
+        was=(frame->quasiact_source!=NULL);
+    
+        frame->quasiact_source=(act ? src : NULL);
+    
+        is=(frame->quasiact_source!=NULL);
+        
+        if(was!=is){
+            frame_quasiactivity_change(frame);
+            if(!REGION_IS_ACTIVE(frame))
+                window_draw((WWindow*)frame, FALSE);
+        }
+    }
 }
 
 
-void frame_activated(WFrame *frame)
+static bool actinact(WRegion *reg, bool act)
 {
-    window_draw((WWindow*)frame, FALSE);
+    WPHolder *returnph=region_get_return(reg);
+    WFrame *frame=NULL;
+    Obj *src=NULL;
+    WRegion *tgt;
+    
+    if(returnph==NULL || pholder_stale(returnph))
+        return FALSE;
+    
+    tgt=pholder_target(returnph);
+
+    frame=OBJ_CAST(tgt, WFrame);
+    
+    if(frame!=NULL){
+        src=(Obj*)returnph;
+    }else{
+        /* Show quasiactivation for stuff detached from
+         * groups contained in the frame as well.
+         */
+        WGroup *grp=OBJ_CAST(tgt, WGroup);
+        if(grp!=NULL){
+            frame=REGION_MANAGER_CHK(grp, WFrame);
+            src=(Obj*)grp;
+        }
+    }
+    
+    if(frame!=NULL)
+        frame_quasiactivation(frame, src, act);
+    
+    return TRUE;
+}
+
+
+static bool activated(WRegion *reg)
+{
+    return actinact(reg, TRUE);
+}
+
+
+static bool inactivated(WRegion *reg)
+{
+    return actinact(reg, FALSE);
+}
+
+
+void ioncore_frame_quasiactivation_notify(WRegion *reg, 
+                                          WRegionNotify how)
+{
+    if(how==ioncore_g.notifies.activated || 
+       how==ioncore_g.notifies.pseudoactivated){
+        activated(reg);
+    }else if(how==ioncore_g.notifies.inactivated ||
+             how==ioncore_g.notifies.pseudoinactivated){
+        inactivated(reg);
+    }else if(how==ioncore_g.notifies.set_return){
+        if(REGION_IS_ACTIVE(reg) || REGION_IS_PSEUDOACTIVE(reg))
+            activated(reg);
+    }else if(how==ioncore_g.notifies.unset_return){
+        if(REGION_IS_ACTIVE(reg) || REGION_IS_PSEUDOACTIVE(reg))
+            inactivated(reg);
+    }
 }
 
 
@@ -555,61 +604,61 @@ void frame_activated(WFrame *frame)
 
 
 static void frame_managed_rqgeom_absolute(WFrame *frame, WRegion *sub,
-                                          int rqflags, const WRectangle *geom_,
+                                          const WRQGeomParams *rq,
                                           WRectangle *geomret)
 {
-    int gravity=ForgetGravity;
-    WRectangle off;
-    WRegion *par;
-    WRectangle geom=*geom_;
-    
     if(!FORWARD_CWIN_RQGEOM(frame)){
         region_managed_rqgeom_absolute_default((WRegion*)frame, sub, 
-                                               rqflags, geom_, geomret);
-        return;
-    }
-
-    /* client window gravity */ {
-        WClientWin *cwin=OBJ_CAST(sub, WClientWin);
+                                               rq, geomret);
+    }else{
+        WRQGeomParams rq2=RQGEOMPARAMS_INIT;
+        int gravity=ForgetGravity;
+        WRectangle off;
+        WRegion *par;
         
-        if(cwin!=NULL && cwin->size_hints.flags&PWinGravity)
-            gravity=cwin->size_hints.win_gravity;
-    }
-    
-    mplex_managed_geom(&frame->mplex, &off);
-    off.x=-off.x;
-    off.y=-off.y;
-    off.w=REGION_GEOM(frame).w-off.w;
-    off.h=REGION_GEOM(frame).h-off.h;
+        rq2.geom=rq->geom;
+        rq2.flags=rq->flags&(REGION_RQGEOM_WEAK_ALL
+                             |REGION_RQGEOM_TRYONLY
+                             |REGION_RQGEOM_ABSOLUTE);
 
-    geom.w=maxof(geom.w+off.w, 0);
-    geom.h=maxof(geom.h+off.h, 0);
+        if(rq->flags&REGION_RQGEOM_GRAVITY)
+            gravity=rq->gravity;
     
-    /*region_size_hints_correct((WRegion*)frame, &(geom.w), &(geom.h), TRUE);*/
-    
-    /* If WEAK_? is set, then geom.(x|y) is root-relative as it was not 
-     * requested by the client and clientwin_handle_configure_request has
-     * no better guess. Otherwise the coordinates are those requested by 
-     * the client (modulo borders/gravity) and we interpret them to be 
-     * root-relative coordinates for this frame modulo gravity.
-     */
-    if(rqflags&REGION_RQGEOM_WEAK_X)
-        geom.x+=off.x;
-    else
-        geom.x+=xgravity_deltax(gravity, -off.x, off.x+off.w);
+        mplex_managed_geom(&frame->mplex, &off);
+        off.x=-off.x;
+        off.y=-off.y;
+        off.w=REGION_GEOM(frame).w-off.w;
+        off.h=REGION_GEOM(frame).h-off.h;
 
-    if(rqflags&REGION_RQGEOM_WEAK_Y)
-        geom.y+=off.y;
-    else
-        geom.y+=xgravity_deltay(gravity, -off.y, off.y+off.h);
+        rq2.geom.w=maxof(rq2.geom.w+off.w, 0);
+        rq2.geom.h=maxof(rq2.geom.h+off.h, 0);
     
-    region_rqgeom((WRegion*)frame, rqflags, &geom, geomret);
+        /*region_size_hints_correct((WRegion*)frame, &(geom.w), &(geom.h), TRUE);*/
     
-    if(geomret!=NULL){
-        geomret->x-=off.x;
-        geomret->y-=off.y;
-        geomret->w-=off.w;
-        geomret->h-=off.h;
+        /* If WEAK_? is set, then geom.(x|y) is root-relative as it was not 
+         * requested by the client and clientwin_handle_configure_request has
+         * no better guess. Otherwise the coordinates are those requested by 
+         * the client (modulo borders/gravity) and we interpret them to be 
+         * root-relative coordinates for this frame modulo gravity.
+         */
+        if(rq->flags&REGION_RQGEOM_WEAK_X)
+            rq2.geom.x+=off.x;
+        else
+            rq2.geom.x+=xgravity_deltax(gravity, -off.x, off.x+off.w);
+	
+        if(rq->flags&REGION_RQGEOM_WEAK_Y)
+            rq2.geom.y+=off.y;
+        else
+            rq2.geom.y+=xgravity_deltay(gravity, -off.y, off.y+off.h);
+        
+        region_rqgeom((WRegion*)frame, &rq2, geomret);
+        
+        if(geomret!=NULL){
+            geomret->x-=off.x;
+            geomret->y-=off.y;
+            geomret->w-=off.w;
+            geomret->h-=off.h;
+        }
     }
 }
 
@@ -617,6 +666,81 @@ static void frame_managed_rqgeom_absolute(WFrame *frame, WRegion *sub,
 /*}}}*/
 
 
+/*{{{ Frame recreate pholder stuff */
+
+
+static WFramedPHolder *frame_make_recreate_pholder(WFrame *frame)
+{
+    WPHolder *ph;
+    WFramedPHolder *fph;
+    WFramedParam fparam=FRAMEDPARAM_INIT;
+    
+    ph=region_make_return_pholder((WRegion*)frame);
+    
+    if(ph==NULL){
+        return NULL;
+    }
+    
+    fparam.mode=frame->mode;
+    
+    fph=create_framedpholder(ph, &fparam);
+    
+    if(fph==NULL){
+        destroy_obj((Obj*)ph);
+        return NULL;
+    }
+    
+    return fph;
+}
+
+
+static void mplex_flatten_phs(WMPlex *mplex)
+{
+    WLListNode *node;
+    WLListIterTmp tmp;
+
+    FOR_ALL_NODES_ON_LLIST(node, mplex->mx_list, tmp){
+        WMPlexPHolder *last=(mplex->mx_phs==NULL ? NULL : mplex->mx_phs->prev);
+        mplex_move_phs(mplex, node, last, NULL);
+    }
+}
+
+
+static void frame_modify_pholders(WFrame *frame)
+{
+    WFramedPHolder *fph;
+    WMPlexPHolder *phs, *ph;
+    
+    mplex_flatten_phs(&frame->mplex);
+    
+    if(frame->mplex.mx_phs==NULL)
+        return;
+    
+    fph=frame_make_recreate_pholder(frame);
+    
+    if(fph==NULL)
+        return;
+    
+    phs=frame->mplex.mx_phs;
+    frame->mplex.mx_phs=NULL;
+    
+    phs->recreate_pholder=fph;
+    
+    for(ph=phs; ph!=NULL; ph=ph->next)
+        watch_reset(&ph->mplex_watch);
+}
+
+
+bool frame_rescue_clientwins(WFrame *frame, WRescueInfo *info)
+{
+    frame_modify_pholders(frame);
+    return mplex_rescue_clientwins(&frame->mplex, info);
+}
+
+    
+/*}}}*/
+
+
 /*{{{ Misc. */
 
 
@@ -624,34 +748,37 @@ bool frame_set_shaded(WFrame *frame, int sp)
 {
     bool set=(frame->flags&FRAME_SHADED);
     bool nset=libtu_do_setparam(sp, set);
-    WRectangle geom=REGION_GEOM(frame);
+    WRQGeomParams rq=RQGEOMPARAMS_INIT;
     GrBorderWidths bdw;
     int h;
 
     if(!XOR(nset, set))
         return nset;
+
+    rq.flags=REGION_RQGEOM_H_ONLY;
+    rq.geom=REGION_GEOM(frame);
         
     if(!nset){
         if(!(frame->flags&FRAME_SAVED_VERT))
             return FALSE;
-        geom.h=frame->saved_h;
+        rq.geom.h=frame->saved_h;
     }else{
         if(frame->barmode==FRAME_BAR_NONE){
             return FALSE;
         }else if(frame->barmode==FRAME_BAR_SHAPED){
-            geom.h=frame->bar_h;
+            rq.geom.h=frame->bar_h;
         }else{
             WRectangle tmp;
             
             frame_border_inner_geom(frame, &tmp);
             
-            geom.h=geom.h-tmp.h;
+            rq.geom.h=rq.geom.h-tmp.h;
         }
     }
     
     frame->flags|=FRAME_SHADED_TOGGLE;
     
-    region_rqgeom((WRegion*)frame, REGION_RQGEOM_H_ONLY, &geom, NULL);
+    region_rqgeom((WRegion*)frame, &rq, NULL);
     
     frame->flags&=~FRAME_SHADED_TOGGLE;
     
@@ -661,7 +788,8 @@ bool frame_set_shaded(WFrame *frame, int sp)
 
 /*EXTL_DOC
  * Set shading state according to the parameter \var{how} 
- * (set/unset/toggle). Resulting state is returned, which may not be
+ * (\codestr{set}, \codestr{unset}, or \codestr{toggle}). 
+ * Resulting state is returned, which may not be
  * what was requested.
  */
 EXTL_EXPORT_AS(WFrame, set_shaded)
@@ -682,47 +810,67 @@ bool frame_is_shaded(WFrame *frame)
 }
 
 
-bool frame_set_numbers(WFrame *frame, int sp)
+/* EXTL_DOC
+ * Is the attribute \var{attr} set?
+ */
+bool frame_is_grattr(WFrame *frame, const char *attr)
 {
-    bool set=frame->flags&FRAME_SHOW_NUMBERS;
+    GrAttr a=stringstore_alloc(attr);
+    bool set=gr_stylespec_isset(&frame->baseattr, a);
+    stringstore_free(a);
+    return set;
+}
+
+
+bool frame_set_grattr(WFrame *frame, GrAttr a, int sp)
+{
+    bool set=gr_stylespec_isset(&frame->baseattr, a);
     bool nset=libtu_do_setparam(sp, set);
     
-    if(XOR(nset, set)){
-        frame->flags^=FRAME_SHOW_NUMBERS;
-        frame_recalc_bar(frame);
-        frame_draw_bar(frame, TRUE);
+    if(XOR(set, nset)){
+        if(nset)
+            gr_stylespec_set(&frame->baseattr, a);
+        else
+            gr_stylespec_unset(&frame->baseattr, a);
+        window_draw((WWindow*)frame, TRUE);
     }
     
-    return frame->flags&FRAME_SHOW_NUMBERS;
+    return nset;
 }
 
 
 /*EXTL_DOC
- * Control whether tabs show numbers (set/unset/toggle). 
- * Resulting state is returned, which may not be what was 
- * requested.
+ * Set extra drawing engine attributes for the frame.
+ * The parameter \var{attr} is the attribute, and \var{how} is
+ * one of \codestr{set}, \codestr{unset}, or \codestr{toggle}.
  */
-EXTL_EXPORT_AS(WFrame, set_numbers)
-bool frame_set_numbers_extl(WFrame *frame, const char *how)
+EXTL_EXPORT_AS(WFrame, set_grattr)
+bool frame_set_grattr_extl(WFrame *frame, const char *attr, const char *how)
 {
-    return frame_set_numbers(frame, libtu_string_to_setparam(how));
+    if(attr!=NULL){
+        GrAttr a=stringstore_alloc(attr);
+        bool ret=frame_set_grattr(frame, a, libtu_string_to_setparam(how));
+        stringstore_free(a);
+        return ret;
+    }else{
+        return FALSE;
+    }
 }
 
 
-/*EXTL_DOC
- * Does \var{frame} show numbers for tabs?
- */
-bool frame_is_numbers(WFrame *frame)
+void frame_managed_notify(WFrame *frame, WRegion *sub, WRegionNotify how)
 {
-    return frame->flags&FRAME_SHOW_NUMBERS;
-}
-
-
-void frame_managed_notify(WFrame *frame, WRegion *sub, const char *how)
-{
-    update_attrs(frame);
-    frame_recalc_bar(frame);
-    frame_draw_bar(frame, FALSE);
+    if(how==ioncore_g.notifies.activated ||
+       how==ioncore_g.notifies.inactivated ||
+       how==ioncore_g.notifies.name ||
+       how==ioncore_g.notifies.activity ||
+       how==ioncore_g.notifies.sub_activity ||
+       how==ioncore_g.notifies.tag){
+       
+        frame_update_attrs(frame);
+        frame_recalc_bar(frame);
+        frame_draw_bar(frame, FALSE);
+    }
 }
 
 
@@ -746,10 +894,18 @@ static void frame_managed_changed(WFrame *frame, int mode, bool sw,
 {
     bool need_draw=TRUE;
     
+    if(mode==MPLEX_CHANGE_REMOVE && (Obj*)reg==frame->quasiact_source){
+        /* Reset indirect quasiactivation through group that
+         * is being removed.
+         */
+        frame->quasiact_source=NULL;
+        frame_quasiactivity_change(frame);
+    }
+    
     if(mode!=MPLEX_CHANGE_SWITCHONLY)
         frame_initialise_titles(frame);
     else
-        update_attrs(frame);
+        frame_update_attrs(frame);
 
     if(sw)
         need_draw=!frame_set_background(frame, FALSE);
@@ -763,34 +919,49 @@ static void frame_managed_changed(WFrame *frame, int mode, bool sw,
 }
 
 
-#define EMPTY_AND_SHOULD_BE_DESTROYED(FRAME) \
-    (DEST_EMPTY(frame) && FRAME_MCOUNT(FRAME)==0 && \
-     !OBJ_IS_BEING_DESTROYED(frame))
-
-
-static void frame_destroy_empty(WFrame *frame)
+WRegion *frame_managed_disposeroot(WFrame *frame, WRegion *reg)
 {
-    if(EMPTY_AND_SHOULD_BE_DESTROYED(frame))
-        destroy_obj((Obj*)frame);
-}
-
-
-void frame_managed_remove(WFrame *frame, WRegion *reg)
-{
-    mplex_managed_remove((WMPlex*)frame, reg);
-    if(EMPTY_AND_SHOULD_BE_DESTROYED(frame)){
-        mainloop_defer_action((Obj*)frame, 
-                              (WDeferredAction*)frame_destroy_empty);
+    if(DEST_EMPTY(frame) &&
+       frame->mplex.mgd!=NULL && 
+       frame->mplex.mgd->reg==reg && 
+       frame->mplex.mgd->next==NULL){
+        WRegion *tmp=region_disposeroot((WRegion*)frame);
+        return (tmp!=NULL ? tmp : reg);
     }
+    
+    return reg;
 }
 
 
 int frame_default_index(WFrame *frame)
 {
-    if(ioncore_g.frame_add_last)
-        return LLIST_INDEX_LAST;
-    else
-        return LLIST_INDEX_AFTER_CURRENT;
+    return ioncore_g.frame_default_index;
+}
+
+
+/*}}}*/
+
+
+/*{{{ prepare_manage_transient */
+
+
+WPHolder *frame_prepare_manage_transient(WFrame *frame,
+                                         const WClientWin *transient,
+                                         const WManageParams *param,
+                                         int unused)
+{
+    /* Transient manager searches should not cross tiled frames
+     * unless explicitly floated.
+     */
+    if(IS_FLOATING_MODE(frame) ||
+       extl_table_is_bool_set(transient->proptab, "float")){
+        return region_prepare_manage_transient_default((WRegion*)frame,
+                                                       transient,
+                                                       param,
+                                                       unused);
+    }else{
+         return NULL;
+    }
 }
 
 
@@ -849,25 +1020,18 @@ WRegion *frame_load(WWindow *par, const WFitParams *fp, ExtlTab tab)
     int mode=FRAME_MODE_UNKNOWN;
     WFrame *frame;
     
-    if(!extl_table_gets_i(tab, "mode", &mode)){
-        #warning "TODO: Remove backwards compatibility hack"
-        char *style=NULL;
-        if(extl_table_gets_s(tab, "frame_style", &style)){
-            if(strcmp(style, "frame-tiled")==0)
-                mode=FRAME_MODE_TILED;
-            else if(strcmp(style, "frame-floating")==0)
-                mode=FRAME_MODE_FLOATING;
-            else if(strcmp(style, "frame-transientcontainer")==0)
-                mode=FRAME_MODE_TRANSIENT;
-            free(style);
-        }
-    }
+    extl_table_gets_i(tab, "mode", &mode);
     
     frame=create_frame(par, fp, mode);
     
     if(frame!=NULL)
         frame_do_load(frame, tab);
     
+    if(DEST_EMPTY(frame) && frame->mplex.mgd==NULL){
+        destroy_obj((Obj*)frame);
+        return NULL;
+    }
+    
     return (WRegion*)frame;
 }
 
@@ -904,14 +1068,21 @@ static DynFunTab frame_dynfuntab[]={
 
     {(DynFun*)region_fitrep,
      (DynFun*)frame_fitrep},
+     
+    {(DynFun*)region_managed_disposeroot,
+     (DynFun*)frame_managed_disposeroot},
 
     {region_managed_rqgeom_absolute, 
      frame_managed_rqgeom_absolute},
 
-    {region_managed_remove, frame_managed_remove},
-    
     {(DynFun*)mplex_default_index,
      (DynFun*)frame_default_index},
+     
+    {(DynFun*)region_prepare_manage_transient,
+     (DynFun*)frame_prepare_manage_transient},
+     
+    {(DynFun*)region_rescue_clientwins,
+     (DynFun*)frame_rescue_clientwins},
     
     END_DYNFUNTAB
 };
