@@ -1,12 +1,9 @@
 /*
  * ion/ioncore/frame.h
  *
- * Copyright (c) Tuomo Valkonen 1999-2006. 
+ * Copyright (c) Tuomo Valkonen 1999-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #ifndef ION_IONCORE_FRAME_H
@@ -36,7 +33,6 @@
 /*#define FRAME_SZH_USEMINMAX 0x1000 */
 /*#define FRAME_FWD_CWIN_RQGEOM 0x2000 */
 
-#define FRAME_SHOW_NUMBERS 0x4000
 
 typedef enum{
     FRAME_MODE_UNKNOWN,
@@ -64,9 +60,11 @@ DECLCLASS(WFrame){
     int saved_x, saved_y;
     
     int tab_dragged_idx;
+    void *quasiact_source;
     
     GrBrush *brush;
     GrBrush *bar_brush;
+    GrStyleSpec baseattr;
     GrTransparency tr_mode;
     GrTextElem *titles;
     int titles_n;
@@ -115,8 +113,17 @@ extern bool frame_is_numbers(WFrame *frame);
 extern int frame_default_index(WFrame *frame);
 
 /* Misc */
-extern void frame_managed_notify(WFrame *frame, WRegion *sub, const char *how);
-extern void frame_managed_remove(WFrame *frame, WRegion *reg);
+extern void frame_managed_notify(WFrame *frame, WRegion *sub, WRegionNotify how);
+extern bool frame_managed_rqdispose(WFrame *frame, WRegion *reg);
+
+extern void ioncore_frame_quasiactivation_notify(WRegion *reg, WRegionNotify how);
+
+extern WPHolder *frame_prepare_manage_transient(WFrame *frame,
+                                                const WClientWin *transient,
+                                                const WManageParams *param,
+                                                int unused);
+
+extern bool frame_rescue_clientwins(WFrame *frame, WRescueInfo *info);
 
 /* Save/load */
 extern ExtlTab frame_get_configuration(WFrame *frame);
