@@ -1,12 +1,9 @@
 /*
  * ion/ioncore/group-cw.h
  *
- * Copyright (c) Tuomo Valkonen 1999-2006. 
+ * Copyright (c) Tuomo Valkonen 1999-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #ifndef ION_IONCORE_GROUPCW_H
@@ -39,7 +36,4 @@ extern WPHolder *groupcw_prepare_manage_transient(WGroupCW *cwg,
 
 extern WRegion *groupcw_load(WWindow *par, const WFitParams *fp, ExtlTab tab);
 
-extern WRegion *create_transient_frame(WWindow *par, 
-                                       const WFitParams *fp);
-
 #endif /* ION_IONCORE_GROUPCW_H */
