@@ -1,12 +1,9 @@
 /*
  * ion/ioncore/groupws.h
  *
- * Copyright (c) Tuomo Valkonen 1999-2006. 
+ * Copyright (c) Tuomo Valkonen 1999-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #ifndef ION_IONCORE_GROUPWS_H
@@ -40,7 +37,6 @@ extern WGroupWS *create_groupws(WWindow *parent, const WFitParams *fp);
 extern bool groupws_init(WGroupWS *ws, WWindow *parent, const WFitParams *fp);
 extern void groupws_deinit(WGroupWS *ws);
 
-extern WRegion *groupws_load_default(WWindow *par, const WFitParams *fp);
 extern WRegion *groupws_load(WWindow *par, const WFitParams *fp, ExtlTab tab);
 
 extern void ioncore_groupws_set(ExtlTab tab);
