@@ -1,12 +1,9 @@
 /*
  * ion/ioncore/groupedpholder.h
  *
- * Copyright (c) Tuomo Valkonen 2005-2006. 
+ * Copyright (c) Tuomo Valkonen 2005-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #ifndef ION_IONCORE_GROUPEDPHOLDER_H
@@ -33,7 +30,9 @@ extern bool groupedpholder_do_goto(WGroupedPHolder *ph);
 
 extern WRegion *groupedpholder_do_target(WGroupedPHolder *ph);
 
-extern bool groupedpholder_do_attach(WGroupedPHolder *ph, int flags,
-                                     WRegionAttachData *data);
+extern WPHolder *groupedpholder_do_root(WGroupedPHolder *ph);
+
+extern WRegion *groupedpholder_do_attach(WGroupedPHolder *ph, int flags,
+                                         WRegionAttachData *data);
 
 #endif /* ION_IONCORE_GROUPEDPHOLDER_H */
