@@ -1,12 +1,9 @@
 /*
  * ion/ioncore/infowin.h
  *
- * Copyright (c) Tuomo Valkonen 1999-2006. 
+ * Copyright (c) Tuomo Valkonen 1999-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #include <string.h>
@@ -19,6 +16,7 @@
 #include "resize.h"
 #include "gr.h"
 #include "event.h"
+#include "strings.h"
 
 
 /*{{{ Init/deinit */
@@ -45,7 +43,8 @@ bool infowin_init(WInfoWin *p, WWindow *parent, const WFitParams *fp,
         goto fail2;
     
     p->brush=NULL;
-    p->attr=NULL;
+    
+    gr_stylespec_init(&p->attr);
     
     infowin_updategr(p);
     
@@ -62,7 +61,8 @@ bool infowin_init(WInfoWin *p, WWindow *parent, const WFitParams *fp,
 
     return TRUE;
 
-fail3:    
+fail3:
+    gr_stylespec_unalloc(&p->attr);
     free(p->style);
 fail2:
     free(p->buffer);
@@ -86,11 +86,6 @@ void infowin_deinit(WInfoWin *p)
         p->buffer=NULL;
     }
 
-    if(p->attr!=NULL){
-        free(p->attr);
-        p->attr=NULL;
-    }
-    
     if(p->style!=NULL){
         free(p->style);
         p->style=NULL;
@@ -101,6 +96,8 @@ void infowin_deinit(WInfoWin *p)
         p->brush=NULL;
     }
     
+    gr_stylespec_unalloc(&p->attr);
+    
     window_deinit(&(p->wwin));
 }
 
@@ -124,7 +121,8 @@ void infowin_draw(WInfoWin *p, bool complete)
     g.h=REGION_GEOM(p).h;
 
     grbrush_begin(p->brush, &g, GRBRUSH_NO_CLEAR_OK);
-    grbrush_draw_textbox(p->brush, &g, p->buffer, p->attr, TRUE);
+    grbrush_init_attr(p->brush, &p->attr);
+    grbrush_draw_textbox(p->brush, &g, p->buffer, TRUE);
     grbrush_end(p->brush);
 }
 
@@ -157,25 +155,9 @@ void infowin_updategr(WInfoWin *p)
 /*{{{ Content-setting */
 
 
-bool infowin_set_attr2(WInfoWin *p, const char *attr1, const char *attr2)
+GrStyleSpec *infowin_stylespec(WInfoWin *p)
 {
-    char *p2=NULL;
-    
-    if(attr1!=NULL){
-        if(attr2==NULL)
-            p2=scopy(attr1);
-        else
-            libtu_asprintf(&p2, "%s-%s", attr1, attr2);
-        if(p2==NULL)
-            return FALSE;
-    }
-    
-    if(p->attr)
-        free(p->attr);
-    
-    p->attr=p2;
-    
-    return TRUE;
+    return &p->attr;
 }
 
 
@@ -188,24 +170,25 @@ static void infowin_do_set_text(WInfoWin *p, const char *str)
 
 static void infowin_resize(WInfoWin *p)
 {
-    int rqflags=REGION_RQGEOM_WEAK_X|REGION_RQGEOM_WEAK_Y;
+    WRQGeomParams rq=RQGEOMPARAMS_INIT;
     const char *str=INFOWIN_BUFFER(p);
-    WRectangle g;
     GrBorderWidths bdw;
     GrFontExtents fnte;
     
-    g.x=REGION_GEOM(p).x;
-    g.y=REGION_GEOM(p).y;
+    rq.flags=REGION_RQGEOM_WEAK_X|REGION_RQGEOM_WEAK_Y;
+    
+    rq.geom.x=REGION_GEOM(p).x;
+    rq.geom.y=REGION_GEOM(p).y;
     
     grbrush_get_border_widths(p->brush, &bdw);
     grbrush_get_font_extents(p->brush, &fnte);
         
-    g.w=bdw.left+bdw.right;
-    g.w+=grbrush_get_text_width(p->brush, str, strlen(str));
-    g.h=fnte.max_height+bdw.top+bdw.bottom;
+    rq.geom.w=bdw.left+bdw.right;
+    rq.geom.w+=grbrush_get_text_width(p->brush, str, strlen(str));
+    rq.geom.h=fnte.max_height+bdw.top+bdw.bottom;
 
-    if(g.w!=REGION_GEOM(p).w || g.h!=REGION_GEOM(p).h)
-        region_rqgeom((WRegion*)p, rqflags, &g, NULL);
+    if(rectangle_compare(&rq.geom, &REGION_GEOM(p))!=RECTANGLE_SAME)
+        region_rqgeom((WRegion*)p, &rq, NULL);
 }
 
 
@@ -213,9 +196,21 @@ static void infowin_resize(WInfoWin *p)
  * Set contents of the info window.
  */
 EXTL_EXPORT_MEMBER
-void infowin_set_text(WInfoWin *p, const char *str)
+void infowin_set_text(WInfoWin *p, const char *str, int maxw)
 {
-    infowin_do_set_text(p, str);
+    bool set=FALSE;
+    
+    if(maxw>0 && p->brush!=NULL){
+        char *tmp=grbrush_make_label(p->brush, str, maxw);
+        if(tmp!=NULL){
+            infowin_do_set_text(p, tmp);
+            free(tmp);
+            set=TRUE;
+        }
+    }
+    
+    if(!set)
+        infowin_do_set_text(p, str);
 
     infowin_resize(p);
     
