@@ -1,12 +1,9 @@
 /*
  * ion/ioncore/infowin.h
  *
- * Copyright (c) Tuomo Valkonen 1999-2006. 
+ * Copyright (c) Tuomo Valkonen 1999-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #ifndef ION_IONCORE_INFOWIN_H
@@ -23,8 +20,8 @@ DECLCLASS(WInfoWin){
     WWindow wwin;
     GrBrush *brush;
     char *buffer;
-    char *attr;
     char *style;
+    GrStyleSpec attr;
 };
 
 #define INFOWIN_BRUSH(INFOWIN) ((INFOWIN)->brush)
@@ -37,8 +34,8 @@ extern WInfoWin *create_infowin(WWindow *parent, const WFitParams *fp,
 
 extern void infowin_deinit(WInfoWin *p);
 
-extern void infowin_set_text(WInfoWin *p, const char *s);
-extern bool infowin_set_attr2(WInfoWin *p, const char *a1, const char *a2);
+extern void infowin_set_text(WInfoWin *p, const char *s, int maxw);
+extern GrStyleSpec *infowin_stylespec(WInfoWin *p);
 
 extern WRegion *infowin_load(WWindow *par, const WFitParams *fp, ExtlTab tab);
 
