@@ -1,12 +1,9 @@
 --
 -- ion/share/ioncore-bindings.lua
 -- 
--- Copyright (c) Tuomo Valkonen 2004-2006.
+-- Copyright (c) Tuomo Valkonen 2004-2007.
 --
--- Ion is free software; you can redistribute it and/or modify it under
--- the terms of the GNU Lesser General Public License as published by
--- the Free Software Foundation; either version 2.1 of the License, or
--- (at your option) any later version.
+-- See the included file LICENSE for details.
 --
 
 local ioncore=_G.ioncore
@@ -140,6 +137,24 @@ function ioncore.kpress_wait(keyspec, cmd, guard)
     return putcmd(cmd, guard, {action = "kpress_wait", kcb = keyspec})
 end
 
+--DOC
+-- Submap enter event for bindings.
+function ioncore.submap_enter(cmd, guard)
+    return putcmd(cmd, guard, {action = "submap_enter"})
+end
+
+--DOC
+-- Submap modifier release event for bindings.
+function ioncore.submap_wait(cmd, guard)
+    return putcmd(cmd, guard, {action = "submap_wait"})
+end
+
+-- DOC
+-- Submap leave event for bindings.
+--function ioncore.submap_leave(cmd, guard)
+--    return putcmd(cmd, guard, {action = "submap_leave"})
+--end
+
 local function mact(act_, kcb_, cmd, guard)
     local st, en, kcb2_, area_=string.find(kcb_, "([^@]*)@(.*)")
     return putcmd(cmd, guard, {
