@@ -1,12 +1,9 @@
 /*
  * ion/ioncore/kbresize.c
  *
- * Copyright (c) Tuomo Valkonen 1999-2006. 
+ * Copyright (c) Tuomo Valkonen 1999-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #include <math.h>
@@ -137,6 +134,8 @@ void moveresmode_accel(WMoveresMode *mode, int *wu, int *hu, int accel_mode)
 static ExtlExportedFn *moveres_safe_fns[]={
     (ExtlExportedFn*)&moveresmode_resize,
     (ExtlExportedFn*)&moveresmode_move,
+    (ExtlExportedFn*)&moveresmode_rqgeom_extl,
+    (ExtlExportedFn*)&moveresmode_geom,
     (ExtlExportedFn*)&moveresmode_finish,
     (ExtlExportedFn*)&moveresmode_cancel,
     NULL
@@ -309,6 +308,33 @@ void moveresmode_move(WMoveresMode *mode, int horizmul, int vertmul)
 }
 
 
+/*EXTL_DOC
+ * Request exact geometry in move/resize mode. For details on parameters,
+ * see \fnref{WRegion.rqgeom}.
+ */
+EXTL_EXPORT_AS(WMoveresMode, rqgeom)
+ExtlTab moveresmode_rqgeom_extl(WMoveresMode *mode, ExtlTab g)
+{
+    WRQGeomParams rq=RQGEOMPARAMS_INIT;
+    WRectangle res;
+    
+    rqgeomparams_from_table(&rq, &mode->geom, g);
+    
+    moveresmode_rqgeom(mode, &rq, &res);
+    
+    return extl_table_from_rectangle(&res);
+}
+
+/*EXTL_DOC
+ * Returns current geometry.
+ */
+EXTL_EXPORT_MEMBER
+ExtlTab moveresmode_geom(WMoveresMode *mode)
+{
+    return extl_table_from_rectangle(&mode->geom);
+}
+
+
 /*EXTL_DOC
  * Return from move/resize mode and apply changes unless opaque
  * move/resize is enabled.
