@@ -1,12 +1,9 @@
 /*
  * ion/ioncore/kbresize.h
  *
- * Copyright (c) Tuomo Valkonen 1999-2006. 
+ * Copyright (c) Tuomo Valkonen 1999-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #ifndef ION_IONCORE_KBRESIZE_H
@@ -28,6 +25,8 @@ extern void moveresmode_move(WMoveresMode *mode,
                              int horizmul, int vertmul);
 extern void moveresmode_resize(WMoveresMode *mode, 
                                int left, int right, int top, int bottom);
+extern ExtlTab moveresmode_geom(WMoveresMode *mode);
+extern ExtlTab moveresmode_rqgeom_extl(WMoveresMode *mode, ExtlTab g);
 extern void moveresmode_accel(WMoveresMode *mode, 
                               int *wu, int *hu, int accel_mode);
 
