@@ -1,12 +1,9 @@
 /*
  * ion/ioncore/llist.h
  *
- * Copyright (c) Tuomo Valkonen 2005-2006. 
+ * Copyright (c) Tuomo Valkonen 2005-2007. 
  * 
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #ifndef ION_IONCORE_LLIST_H
@@ -49,7 +46,8 @@ extern WLListNode *llist_index_to_after(WLListNode *list,
                                         int index);
 extern void llist_unlink(WLListNode **list, WLListNode *node);
 
-#define LLIST_INDEX_LAST (-1)
-#define LLIST_INDEX_AFTER_CURRENT (-2)
+#define LLIST_INDEX_LAST              (-1)
+#define LLIST_INDEX_AFTER_CURRENT     (-2)
+#define LLIST_INDEX_AFTER_CURRENT_ACT (-3)
 
 #endif /* ION_IONCORE_LLIST_H */
