@@ -1,12 +1,9 @@
 /*
  * ion/ioncore/mplex.h
  *
- * Copyright (c) Tuomo Valkonen 1999-2006. 
+ * Copyright (c) Tuomo Valkonen 1999-2007. 
  * 
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #ifndef ION_IONCORE_MPLEX_H
@@ -20,6 +17,8 @@
 #include "manage.h"
 #include "rectangle.h"
 #include "pholder.h"
+#include "sizepolicy.h"
+
 
 #define MPLEX_ADD_TO_END 0x0001
 #define MPLEX_MANAGED_UNVIEWABLE 0x0002
@@ -27,12 +26,17 @@
 #define MPLEX_MGD_UNVIEWABLE(MPLEX) \
     ((MPLEX)->flags&MPLEX_MANAGED_UNVIEWABLE)
 
-#define MPLEX_ATTACH_SWITCHTO 0x01
-#define MPLEX_ATTACH_L2 0x02
-#define MPLEX_ATTACH_L2_PASSIVE 0x04
-#define MPLEX_ATTACH_L2_GEOM 0x08
-#define MPLEX_ATTACH_L2_HIDDEN 0x10
-#define MPLEX_ATTACH_L2_SEMIMODAL 0x20
+
+#define MPLEX_ATTACH_SWITCHTO     0x0001 /* switch to region */
+#define MPLEX_ATTACH_UNNUMBERED   0x0002 /* do not put on mut.ex list */
+#define MPLEX_ATTACH_HIDDEN       0x0004 /* should be hidden */
+#define MPLEX_ATTACH_PSEUDOMODAL  0x0004 /* pseudomodal (if modal) */
+#define MPLEX_ATTACH_LEVEL        0x0010 /* level field set */
+#define MPLEX_ATTACH_GEOM         0x0020 /* geometry field is set */
+#define MPLEX_ATTACH_SIZEPOLICY   0x0040 /* size policy field is set */
+#define MPLEX_ATTACH_INDEX        0x0080 /* index field is set */
+#define MPLEX_ATTACH_WHATEVER     0x0100 /* set REGION_FIT_WHATEVER */
+
 
 enum{
     MPLEX_CHANGE_SWITCHONLY=0,
@@ -53,7 +57,6 @@ enum{
 INTRSTRUCT(WMPlexSTDispInfo);
 INTRSTRUCT(WMPlexChangedParams);
 INTRSTRUCT(WMPlexAttachParams);
-INTRSTRUCT(WLListNode); /* in mplexp.h */
 
 
 DECLSTRUCT(WMPlexSTDispInfo){
@@ -70,10 +73,14 @@ DECLSTRUCT(WMPlexChangedParams){
 };
 
 
+#define MPLEXATTACHPARAMS_INIT {0, 0, {0, 0, 0, 0}, 0, 0}
+
 DECLSTRUCT(WMPlexAttachParams){
     int flags;
     int index;
-    WRectangle l2geom;
+    WRectangle geom;
+    WSizePolicy szplcy;
+    uint level;
 };
 
 
@@ -81,15 +88,12 @@ DECLCLASS(WMPlex){
     WWindow win;
     int flags;
     
-    int l1_count;
-    WLListNode *l1_list;
-    WLListNode *l1_current;
-    WMPlexPHolder *l1_phs;
+    WStacking *mgd;
     
-    int l2_count;
-    WLListNode *l2_list;
-    WLListNode *l2_current;
-    WMPlexPHolder *l2_phs;
+    int mx_count;
+    WLListNode *mx_current;
+    WLListNode *mx_list;
+    WMPlexPHolder *mx_phs;
     
     Watch stdispwatch;
     WMPlexSTDispInfo stdispinfo;
@@ -101,8 +105,8 @@ DECLCLASS(WMPlex){
 extern WMPlex *create_mplex(WWindow *parent, const WFitParams *fp);
 extern bool mplex_init(WMPlex *mplex, WWindow *parent,
                        const WFitParams *fp);
-extern bool mplex_do_init(WMPlex *mplex, WWindow *parent, Window win,
-                          const WFitParams *fp, bool create);
+extern bool mplex_do_init(WMPlex *mplex, WWindow *parent, 
+                          const WFitParams *fp, Window win);
 extern void mplex_deinit(WMPlex *mplex);
 
 /* Resize and reparent */
@@ -120,61 +124,65 @@ extern void mplex_unmap(WMPlex *mplex);
 
 extern WRegion *mplex_attach_simple(WMPlex *mplex, WRegion *reg, 
                                     int flags);
-extern WRegion *mplex_attach_hnd(WMPlex *mplex, WRegionAttachHandler *hnd,
-                                 void *hnd_param, int flags);
 
 extern WRegion *mplex_attach(WMPlex *mplex, WRegion *reg, ExtlTab param);
+extern WRegion *mplex_attach_new_(WMPlex *mplex, WMPlexAttachParams *partmpl,
+                                  int mask, ExtlTab param);
 extern WRegion *mplex_attach_new(WMPlex *mplex, ExtlTab param);
 
-extern WRegion *mplex_do_attach(WMPlex *mplex, WRegionAttachHandler *hnd,
-                                void *hnd_param, WMPlexAttachParams *param);
-extern WLListNode *mplex_do_attach_after(WMPlex *mplex, 
-                                            WLListNode *after, 
-                                            WMPlexAttachParams *param,
-                                            WRegionAttachHandler *hnd,
-                                            void *hnd_param);
-
-extern void mplex_attach_tagged(WMPlex *mplex);
+extern WRegion *mplex_do_attach_pholder(WMPlex *mplex, WMPlexPHolder *ph,
+                                        WRegionAttachData *data);
+extern WRegion *mplex_do_attach(WMPlex *mplex, WMPlexAttachParams *param,
+                                WRegionAttachData *data);
+extern WRegion *mplex_do_attach_new(WMPlex *mplex, WMPlexAttachParams *param,
+                                    WRegionCreateFn *fn, void *fn_param);
 
 extern void mplex_managed_remove(WMPlex *mplex, WRegion *reg);
 extern void mplex_child_removed(WMPlex *mplex, WRegion *sub);
 
-extern bool mplex_rescue_clientwins(WMPlex *mplex, WPHolder *ph);
+extern bool mplex_rescue_clientwins(WMPlex *mplex, WRescueInfo *info);
 
 extern WPHolder *mplex_prepare_manage(WMPlex *mplex, const WClientWin *cwin,
                                       const WManageParams *param, int redir);
 
 /* Switch */
 
-extern bool mplex_managed_goto(WMPlex *mplex, WRegion *sub, bool cfocus);
+extern bool mplex_managed_prepare_focus(WMPlex *mplex, WRegion *sub, 
+                                        int flags, WPrepareFocusResult *res);
+extern bool mplex_do_prepare_focus(WMPlex *mplex, WStacking *disp, 
+                                   WStacking *sub, int flags, 
+                                   WPrepareFocusResult *res);
+
 extern void mplex_switch_nth(WMPlex *mplex, uint n);
 extern void mplex_switch_next(WMPlex *mplex);
 extern void mplex_switch_prev(WMPlex *mplex);
-extern bool mplex_l2_is_hidden(WMPlex *mplex, WRegion *reg);
-extern bool mplex_l2_set_hidden(WMPlex *mplex, WRegion *reg, bool sp);
-extern bool mplex_l2_is_passive(WMPlex *mplex, WRegion *reg);
-extern bool mplex_l2_set_passive(WMPlex *mplex, WRegion *reg, bool sp);
+extern bool mplex_is_hidden(WMPlex *mplex, WRegion *reg);
+extern bool mplex_set_hidden(WMPlex *mplex, WRegion *reg, int sp);
 
 /* Focus */
 
 extern void mplex_do_set_focus(WMPlex *mplex, bool warp);
-extern void mplex_managed_activated(WMPlex *mplex, WRegion *reg);
+
+/* Stacking */
+
+extern bool mplex_managed_rqorder(WMPlex *mplex, WRegion *sub, 
+                                  WRegionOrder order);
 
 /* Misc */
 
 extern WRegion *mplex_current(WMPlex *mplex);
-extern bool mplex_may_destroy(WMPlex *mplex);
 
-extern int mplex_layer(WMPlex *mplex, WRegion *reg);
-extern int mplex_lcount(WMPlex *mplex, uint l);
-extern WRegion *mplex_lnth(WMPlex *mplex, uint l, uint n);
-extern ExtlTab mplex_llist(WMPlex *mplex, uint l);
-extern WRegion *mplex_lcurrent(WMPlex *mplex, uint l);
+extern bool mplex_managed_i(WMPlex *mplex, ExtlFn iterfn);
+
+extern int mplex_mx_count(WMPlex *mplex);
+extern WRegion *mplex_mx_nth(WMPlex *mplex, uint n);
+extern bool mplex_mx_i(WMPlex *mplex, ExtlFn iterfn);
+extern WRegion *mplex_mx_current(WMPlex *mplex);
 
 extern void mplex_call_changed_hook(WMPlex *mplex, WHook *hook, 
                                     int mode, bool sw, WRegion *reg);
 
-extern WLListNode *mplex_find_node(WMPlex *mplex, WRegion *reg);
+extern void mplex_remanage_stdisp(WMPlex *mplex);
 
 /* Dynfuns */
 
@@ -201,5 +209,33 @@ extern void mplex_get_stdisp(WMPlex *mplex, WRegion **stdisp,
 extern WRegion *mplex_set_stdisp_extl(WMPlex *mplex, ExtlTab t);
 extern ExtlTab mplex_get_stdisp_extl(WMPlex *mplex);
 
+DYNFUN void region_manage_stdisp(WRegion *reg, WRegion *stdisp, 
+                                 const WMPlexSTDispInfo *info);
+DYNFUN void region_unmanage_stdisp(WRegion *reg, bool permanent, bool nofocus);
+
+/* Stacking list stuff */
+
+typedef WStackingIterTmp WMPlexIterTmp;
+
+extern void mplex_iter_init(WMPlexIterTmp *tmp, WMPlex *ws);
+extern WRegion *mplex_iter(WMPlexIterTmp *tmp);
+extern WStacking *mplex_iter_nodes(WMPlexIterTmp *tmp);
+
+extern WStacking *mplex_get_stacking(WMPlex *ws);
+extern WStacking **mplex_get_stackingp(WMPlex *ws);
+
+#define FOR_ALL_MANAGED_BY_MPLEX(MPLEX, VAR, TMP) \
+    for(mplex_iter_init(&(TMP), MPLEX),           \
+         VAR=mplex_iter(&(TMP));                  \
+        VAR!=NULL;                                \
+        VAR=mplex_iter(&(TMP)))
+
+#define FOR_ALL_NODES_IN_MPLEX(MPLEX, VAR, TMP) \
+    for(mplex_iter_init(&(TMP), MPLEX),         \
+         VAR=mplex_iter_nodes(&(TMP));          \
+        VAR!=NULL;                              \
+        VAR=mplex_iter_nodes(&(TMP)))
+
+extern WStacking *mplex_find_stacking(WMPlex *mplex, WRegion *reg);
 
 #endif /* ION_IONCORE_MPLEX_H */
