@@ -1,12 +1,9 @@
 /*
  * ion/ioncore/names.h
  *
- * Copyright (c) Tuomo Valkonen 1999-2002. 
+ * Copyright (c) Tuomo Valkonen 1999-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #ifndef ION_IONCORE_NAMES_H
@@ -39,11 +36,12 @@ extern void region_unregister(WRegion *reg);
 extern void region_do_unuse_name(WRegion *reg, bool insert_unnamed);
 
 extern const char *region_name(WRegion *reg);
+DYNFUN const char *region_displayname(WRegion *reg);
 
 extern char *region_make_label(WRegion *reg, int maxw, GrBrush *brush);
 
-extern ExtlTab ioncore_region_list(const char *typenam);
-extern ExtlTab ioncore_clientwin_list();
+extern bool ioncore_region_i(ExtlFn fn, const char *typenam);
+extern bool ioncore_clientwin_i(ExtlFn fn);
 extern WRegion *ioncore_lookup_region(const char *cname, const char *typenam);
 extern WClientWin *ioncore_lookup_clientwin(const char *cname);
 
