@@ -1,12 +1,9 @@
 /*
  * ion/ioncore/netwm.h
  *
- * Copyright (c) Tuomo Valkonen 1999-2005. 
+ * Copyright (c) Tuomo Valkonen 1999-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #ifndef ION_IONCORE_NETWM_H
@@ -14,6 +11,7 @@
 
 #include "common.h"
 #include "rootwin.h"
+#include "screen.h"
 
 #define _NET_WM_STATE_REMOVE        0    /* remove/unset property */
 #define _NET_WM_STATE_ADD           1    /* add/set property */
@@ -22,7 +20,7 @@
 extern void netwm_init();
 extern void netwm_init_rootwin(WRootWin *rw);
 
-extern int netwm_check_initial_fullscreen(WClientWin *cwin, bool switchto);
+extern WScreen *netwm_check_initial_fullscreen(WClientWin *cwin);
 extern void netwm_update_state(WClientWin *cwin);
 extern void netwm_delete_state(WClientWin *cwin);
 extern void netwm_set_active(WRegion *reg);
