@@ -1,12 +1,9 @@
 /*
  * ion/ioncore/pholder.h
  *
- * Copyright (c) Tuomo Valkonen 2005-2006. 
+ * Copyright (c) Tuomo Valkonen 2005-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #ifndef ION_IONCORE_PHOLDER_H
@@ -31,18 +28,26 @@ DECLCLASS(WPHolder){
 extern bool pholder_init(WPHolder *ph);
 extern void pholder_deinit(WPHolder *ph);
 
-DYNFUN bool pholder_do_attach(WPHolder *ph, int flags,
-                              WRegionAttachData *data);
+DYNFUN WRegion *pholder_do_attach(WPHolder *ph, int flags,
+                                  WRegionAttachData *data);
 
-extern bool pholder_attach_(WPHolder *ph, int flags, 
-                            WRegionAttachData *data);
+extern WRegion *pholder_attach_(WPHolder *ph, int flags, 
+                                WRegionAttachData *data);
 
 extern bool pholder_attach(WPHolder *ph, int flags, WRegion *reg);
 
+extern bool pholder_attach_mcfgoto(WPHolder *ph, int flags, WRegion *reg);
+
 DYNFUN WRegion *pholder_do_target(WPHolder *ph);
 
 extern WRegion *pholder_target(WPHolder *ph);
 
+extern WPHolder *pholder_do_root(WPHolder *ph);
+
+extern WPHolder *pholder_root(WPHolder *ph);
+
+extern bool pholder_stale(WPHolder *ph);
+
 DYNFUN bool pholder_do_check_reparent(WPHolder *ph, WRegion *reg);
 
 extern bool pholder_check_reparent(WPHolder *ph, WRegion *reg);
