@@ -1,12 +1,9 @@
 /*
  * ion/ioncore/pointer.c
  *
- * Copyright (c) Tuomo Valkonen 1999-2006. 
+ * Copyright (c) Tuomo Valkonen 1999-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #include "common.h"
@@ -137,7 +134,8 @@ static void call_button(WBinding *binding, XButtonEvent *ev)
         return;
 
     p_curr_event=(XEvent*)ev;
-    extl_call(binding->func, "oo", NULL, p_reg, p_subreg);
+    extl_call(binding->func, "ooo", NULL, p_reg, p_subreg, 
+              (p_reg!=NULL ? p_reg->active_sub : NULL));
     p_curr_event=NULL;
 }
 
