@@ -1,12 +1,9 @@
 /*
  * ion/ioncore/property.c
  *
- * Copyright (c) Tuomo Valkonen 1999-2006. 
+ * Copyright (c) Tuomo Valkonen 1999-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #include <X11/Xmd.h>
@@ -218,13 +215,8 @@ void xwindow_set_text_property(Window win, Atom a, const char **ptr, int n)
     if(!ioncore_g.use_mb){
         st=XStringListToTextProperty((char **)&ptr, n, &prop);
     }else{
-#ifdef X_HAVE_UTF8_STRING        
-        st=XmbTextListToTextProperty(ioncore_g.dpy, (char **)ptr, n,
-                                     XUTF8StringStyle, &prop);
-#else        
         st=XmbTextListToTextProperty(ioncore_g.dpy, (char **)ptr, n,
                                      XTextStyle, &prop);
-#endif        
         st=!st;
     }
     
@@ -340,7 +332,7 @@ static bool get_mode(const char *mode, int *m)
 
 /*EXTL_DOC
  * Modify a window property. The \var{mode} is one of
- * \code{"replace"}, \code{"prepend"} or \code{"append"}, and format
+ * \codestr{replace}, \codestr{prepend} or \codestr{append}, and format
  * is either 8, 16 or 32. Also see \fnref{ioncore.x_get_window_property}
  * and the \code{XChangeProperty}(3) manual page.
  */
@@ -382,8 +374,7 @@ void ioncore_x_delete_property(int win, int atom)
 
 
 /*EXTL_DOC
- * Get a text property for a window (\code{STRING}, \code{COMPOUND_TEXT},
- * or \code{UTF8_STRING} property converted). The fields in the returned
+ * Get a text property for a window. The fields in the returned
  * table (starting from 1) are the null-separated parts of the property.
  * See the \code{XGetTextProperty}(3) manual page for more information.
  */
