@@ -1,12 +1,9 @@
 /*
  * ion/ioncore/resize.h
  *
- * Copyright (c) Tuomo Valkonen 1999-2006. 
+ * Copyright (c) Tuomo Valkonen 1999-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #ifndef ION_IONCORE_RESIZE_H
@@ -16,6 +13,7 @@
 #include "frame.h"
 #include "infowin.h"
 #include "rectangle.h"
+#include "sizehint.h"
 
 
 /* To make it easier for region_managed_rqgeom handlers, the geom
@@ -31,6 +29,8 @@
 #define REGION_RQGEOM_WEAK_H    0x0008
 #define REGION_RQGEOM_WEAK_ALL  0x000f
 #define REGION_RQGEOM_TRYONLY   0x0010
+#define REGION_RQGEOM_ABSOLUTE  0x0020
+#define REGION_RQGEOM_GRAVITY   0x0040
 
 #define REGION_RQGEOM_NORMAL    0
 #define REGION_RQGEOM_VERT_ONLY (REGION_RQGEOM_WEAK_X|REGION_RQGEOM_WEAK_W)
@@ -42,12 +42,23 @@
 #define REGION_ORIENTATION_HORIZONTAL 1
 #define REGION_ORIENTATION_VERTICAL 2
 
+
+#define RQGEOMPARAMS_INIT {{0, 0, 0, 0}, 0, 0}
+
+
+DECLSTRUCT(WRQGeomParams){
+    WRectangle geom;
+    int flags;
+    int gravity;
+};
+
+
 typedef void WDrawRubberbandFn(WRootWin *rw, const WRectangle *geom);
 
 
 DECLCLASS(WMoveresMode){
     Obj obj;
-    XSizeHints hints;
+    WSizeHints hints;
     int dx1, dx2, dy1, dy2;
     WRectangle origgeom;
     WRectangle geom;
@@ -62,6 +73,7 @@ DECLCLASS(WMoveresMode){
     WInfoWin *infowin;
 };
 
+
 extern WMoveresMode *region_begin_resize(WRegion *reg, 
                                          WDrawRubberbandFn *rubfn,
                                          bool cumulative);
@@ -78,6 +90,8 @@ extern void moveresmode_delta_resize(WMoveresMode *mode,
                                      WRectangle *rret);
 extern void moveresmode_delta_move(WMoveresMode *mode, 
                                    int dx, int dy, WRectangle *rret);
+extern void moveresmode_rqgeom(WMoveresMode *mode, WRQGeomParams *rq, 
+                               WRectangle *rret);
 extern bool moveresmode_do_end(WMoveresMode *mode, bool apply);
 extern WRegion *moveresmode_target(WMoveresMode *mode);
 
@@ -88,36 +102,38 @@ extern WMoveresMode *moveres_mode(WRegion *reg);
 /* Note: even if REGION_RQGEOM_(X|Y|W|H) are not all specified, the
  * geom parameter should contain a proper geometry!
  */
-DYNFUN void region_managed_rqgeom(WRegion *reg, WRegion *sub, int flags, 
-                                  const WRectangle *geom,
+DYNFUN void region_managed_rqgeom(WRegion *reg, WRegion *sub, 
+                                  const WRQGeomParams *rqgp,
                                   WRectangle *geomret);
+DYNFUN void region_managed_rqgeom_absolute(WRegion *reg, WRegion *sub, 
+                                           const WRQGeomParams *rqgp,
+                                           WRectangle *geomret);
 
-extern void region_rqgeom(WRegion *reg, int flags, 
-                          const WRectangle *geom,
+extern void region_rqgeom(WRegion *reg, const WRQGeomParams *rqgp,
                           WRectangle *geomret);
 
 /* Implementation for regions that do not allow subregions to resize
  * themselves; default is to give the size the region wants.
  */
 extern void region_managed_rqgeom_unallow(WRegion *reg, WRegion *sub,
-                                          int flags, 
-                                          const WRectangle *geom,
+                                          const WRQGeomParams *rq,
                                           WRectangle *geomret);
 /* default */
 extern void region_managed_rqgeom_allow(WRegion *reg, WRegion *sub,
-                                        int flags, 
-                                        const WRectangle *geom, 
+                                        const WRQGeomParams *rq,
                                         WRectangle *geomret);
 
-/* This function expects a root-relative geometry and the client expects
- * the gravity size hint be taken into account.
- */
-DYNFUN void region_rqgeom_clientwin(WRegion *reg, WClientWin *cwin,
-                                    int flags, const WRectangle *geom);
+extern void region_managed_rqgeom_absolute_default(WRegion *reg, WRegion *sub,
+                                                   const WRQGeomParams *rq,
+                                                   WRectangle *geomret);
 
-DYNFUN void region_size_hints(WRegion *reg, XSizeHints *hints_ret);
+
+DYNFUN void region_size_hints(WRegion *reg, WSizeHints *hints_ret);
 DYNFUN int region_orientation(WRegion *reg);
 
+extern void region_size_hints_correct(WRegion *reg, 
+                                      int *wp, int *hp, bool min);
+
 extern uint region_min_h(WRegion *reg);
 extern uint region_min_w(WRegion *reg);
 
@@ -126,4 +142,11 @@ extern void frame_maximize_horiz(WFrame *frame);
 
 extern void region_convert_root_geom(WRegion *reg, WRectangle *geom);
 
+extern void region_absolute_geom_to_parent(WRegion *reg, 
+                                           const WRectangle *rgeom,
+                                           WRectangle *pgeom);
+
+extern void rqgeomparams_from_table(WRQGeomParams *rq, 
+                                    const WRectangle *origg, ExtlTab g);
+
 #endif /* ION_IONCORE_RESIZE_H */
