@@ -1,12 +1,9 @@
 /*
  * ion/ioncore/saveload.c
  *
- * Copyright (c) Tuomo Valkonen 1999-2005. 
+ * Copyright (c) Tuomo Valkonen 1999-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #include <string.h>
@@ -14,18 +11,19 @@
 #include <unistd.h>
 
 #include <libtu/objp.h>
+#include <libextl/readconfig.h>
+#include <libextl/extl.h>
 
 #include "common.h"
 #include "global.h"
 #include "region.h"
-#include <libextl/readconfig.h>
 #include "screen.h"
 #include "saveload.h"
 #include "names.h"
 #include "attach.h"
 #include "reginfo.h"
-#include <libextl/extl.h>
 #include "extlconv.h"
+#include "group-ws.h"
 
 
 static bool loading_layout=FALSE;
@@ -80,10 +78,15 @@ WRegion *create_region_load(WWindow *par, const WFitParams *fp,
     WRegionLoadCreateFn* fn=NULL;
     WRegClassInfo *info=NULL;
     WRegion *reg=NULL;
+    bool grouped=FALSE;
+    char *grouped_name=NULL;
     
     if(!extl_table_gets_s(tab, "type", &objclass))
         return NULL;
-
+    
+    if(objclass==NULL)
+        return NULL;
+    
     info=ioncore_lookup_regclass(objclass, FALSE);
     if(info!=NULL)
         fn=info->lc_fn;
@@ -154,10 +157,34 @@ ExtlTab region_get_base_configuration(WRegion *reg)
 }
 
 
+static bool get_config_clientwins=TRUE;
+
+
 ExtlTab region_get_configuration(WRegion *reg)
 {
     ExtlTab tab=extl_table_none();
-    CALL_DYN_RET(tab, ExtlTab, region_get_configuration, reg, (reg));
+    if(get_config_clientwins || !OBJ_IS(reg, WClientWin)){
+        CALL_DYN_RET(tab, ExtlTab, region_get_configuration, reg, (reg));
+    }
+    return tab;
+}
+
+
+/*EXTL_DOC
+ * Get configuration tree. If \var{clientwins} is unset, client windows
+ * are filtered out.
+ */
+EXTL_EXPORT_AS(WRegion, get_configuration)
+ExtlTab region_get_configuration_extl(WRegion *reg, bool clientwins)
+{
+    ExtlTab tab;
+    
+    get_config_clientwins=clientwins;
+    
+    tab=region_get_configuration(reg);
+    
+    get_config_clientwins=TRUE;
+    
     return tab;
 }
 
@@ -193,6 +220,14 @@ bool ioncore_init_layout()
     FOR_ALL_SCREENS(scr){
         ExtlTab scrtab=extl_table_none();
         bool scrok=FALSE;
+        
+        /* Potential Xinerama or such support should set the screen ID
+         * of the root window to less than zero, and number its own
+         * fake screens up from 0.
+         */
+        if(screen_id(scr)<0)
+            continue;
+
         if(ok)
             scrok=extl_table_geti_t(tab, screen_id(scr), &scrtab);
         
@@ -245,7 +280,14 @@ bool ioncore_save_layout()
         return FALSE;
     
     FOR_ALL_SCREENS(scr){
-        ExtlTab scrtab=region_get_configuration((WRegion*)scr);
+        ExtlTab scrtab;
+        
+        /* See note above */
+        if(screen_id(scr)<0)
+            continue;
+            
+        scrtab=region_get_configuration((WRegion*)scr);
+        
         if(scrtab==extl_table_none()){
             warn(TR("Unable to get configuration for screen %d."),
                  screen_id(scr));
