@@ -1,12 +1,9 @@
 /*
  * ion/ioncore/xwindow.c
  *
- * Copyright (c) Tuomo Valkonen 1999-2006. 
+ * Copyright (c) Tuomo Valkonen 1999-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #include <string.h>
@@ -112,10 +109,10 @@ void xwindow_set_cursor(Window win, int cursor)
 bool xwindow_pointer_pos(Window rel, int *px, int *py)
 {
     Window win=None, realroot=None;
-    int wx=0, wy=0;
+    int rx=0, ry=0;
     uint mask=0;
     return XQueryPointer(ioncore_g.dpy, rel, &realroot, &win,
-                         px, py, &wx, &wy, &mask);
+                         &rx, &ry, px, py, &mask);
 }
 
 /*}}}*/
