@@ -30,8 +30,16 @@ void libtu_init(const char *argv0)
 }
 
 
-const char *prog_execname()
+const char *libtu_progname()
 {
     return progname;
 }
 
+
+const char *libtu_progbasename()
+{
+    const char *s=strrchr(progname, '/');
+    
+    return (s==NULL ? progname : s+1);
+}
+
