@@ -2,7 +2,7 @@
  * Ion dock module
  * Copyright (C) 2003 Tom Payne
  * Copyright (C) 2003 Per Olofsson
- * Copyright (C) 2004-2006 Tuomo Valkonen
+ * Copyright (C) 2004-2007 Tuomo Valkonen
  *
  * by Tom Payne <ion@tompayne.org>
  * based on code by Per Olofsson <pelle@dsv.su.se>
@@ -806,15 +806,20 @@ static void dock_managed_rqgeom_(WDock *dock, WRegion *reg, int flags,
     
     /* Fit dock to new geom if required */
     if(!(flags&REGION_RQGEOM_TRYONLY)){
-        int rqgeomflags=REGION_RQGEOM_WEAK_X|REGION_RQGEOM_WEAK_Y;
+        WRQGeomParams rq=RQGEOMPARAMS_INIT;
         
         dock_set_minmax(dock, grow, &border_dock_geom);
         
         if(just_update_minmax)
             return;
         
+        rq.flags=REGION_RQGEOM_WEAK_X|REGION_RQGEOM_WEAK_Y;
+        rq.geom=border_dock_geom;
+
         dock->arrange_called=FALSE;
-        region_rqgeom((WRegion*)dock, rqgeomflags, &border_dock_geom, NULL);
+        
+        region_rqgeom((WRegion*)dock, &rq, NULL);
+        
         if(!dock->arrange_called)
             dock_arrange_dockapps(dock, &REGION_GEOM(dock), NULL, NULL);
         
@@ -829,10 +834,11 @@ static void dock_managed_rqgeom_(WDock *dock, WRegion *reg, int flags,
     }
 }
 
-static void dock_managed_rqgeom(WDock *dock, WRegion *reg, int flags,
-                                const WRectangle *geom, WRectangle *geomret)
+static void dock_managed_rqgeom(WDock *dock, WRegion *reg, 
+                                const WRQGeomParams *rq, 
+                                WRectangle *geomret)
 {
-    dock_managed_rqgeom_(dock, reg, flags, geom, geomret, FALSE);
+    dock_managed_rqgeom_(dock, reg, rq->flags, &rq->geom, geomret, FALSE);
 }
 
 
@@ -899,7 +905,7 @@ static void dock_draw(WDock *dock, bool complete)
         {
             WRectangle geom=REGION_GEOM(dock);
             geom.x=geom.y=0;
-            grbrush_draw_border(dock->brush, &geom, "dock");
+            grbrush_draw_border(dock->brush, &geom);
         }
         break;
     case DOCK_OUTLINE_STYLE_EACH:
@@ -907,8 +913,7 @@ static void dock_draw(WDock *dock, bool complete)
             WDockApp *dockapp;
             for(dockapp=dock->dockapps; dockapp!=NULL;
                 dockapp=dockapp->next){
-                grbrush_draw_border(dock->brush, &dockapp->tile_geom, 
-                                    "dock");
+                grbrush_draw_border(dock->brush, &dockapp->tile_geom);
             }
         }
         break;
@@ -924,7 +929,7 @@ static void dock_draw(WDock *dock, bool complete)
 EXTL_EXPORT_MEMBER
 void dock_resize(WDock *dock)
 {
-    dock_managed_rqgeom(dock, NULL, 0, NULL, NULL);
+    dock_managed_rqgeom_(dock, NULL, 0, NULL, NULL, FALSE);
     dock_draw(dock, TRUE);
     
 }
@@ -1150,18 +1155,6 @@ static void dock_deinit(WDock *dock)
 }
 
 
-bool dock_may_destroy(WDock *dock)
-{
-    if(dock->dockapps!=NULL){
-        warn_obj(modname, "Dock \"%s\" is still managing other objects "
-                " -- refusing to close.", region_name((WRegion*)dock));
-        return FALSE;
-    }
-
-    return TRUE;
-}
-
-
 EXTL_EXPORT
 WDock *mod_dock_create(ExtlTab tab)
 {
@@ -1252,16 +1245,17 @@ WDock *mod_dock_create(ExtlTab tab)
     
     /* Final setup */    
     if(floating){
-        WRectangle dg;
+        WRQGeomParams rq=RQGEOMPARAMS_INIT;
         const WRectangle *pg=&REGION_GEOM(screen);
         
         /* Just calculate real min/max size */
         dock_managed_rqgeom_(dock, NULL, 0, NULL, NULL, TRUE);
         
-        dg.w=minof(dock->min_w, pg->w);
-        dg.h=minof(dock->min_h, pg->h);
-        calc_dock_pos(&dg, pg, dock->pos);
-        region_rqgeom((WRegion*)dock, 0, &dg, NULL);
+        rq.geom.w=minof(dock->min_w, pg->w);
+        rq.geom.h=minof(dock->min_h, pg->h);
+        calc_dock_pos(&rq.geom, pg, dock->pos);
+        
+        region_rqgeom((WRegion*)dock, &rq, NULL);
         
         return dock;
     }else{
@@ -1379,9 +1373,9 @@ static bool dock_do_attach_final(WDock *dock, WRegion *reg, void *unused)
     region_set_manager(reg, (WRegion*)dock);
     
     geom=REGION_GEOM(reg);
-    dock_managed_rqgeom(dock, reg, 
-                        REGION_RQGEOM_WEAK_X|REGION_RQGEOM_WEAK_Y,
-                        &geom, NULL);
+    dock_managed_rqgeom_(dock, reg, 
+                         REGION_RQGEOM_WEAK_X|REGION_RQGEOM_WEAK_Y,
+                         &geom, NULL, FALSE);
     
     region_map(reg);
 
@@ -1390,7 +1384,7 @@ static bool dock_do_attach_final(WDock *dock, WRegion *reg, void *unused)
 
 
 
-static bool dock_do_attach(WDock *dock, WRegionAttachData *data)
+static WRegion *dock_do_attach(WDock *dock, WRegionAttachData *data)
 {
     WFitParams fp;
     dock_get_tile_size(dock, &(fp.g));
@@ -1398,10 +1392,9 @@ static bool dock_do_attach(WDock *dock, WRegionAttachData *data)
     fp.g.y=0;
     fp.mode=REGION_FIT_WHATEVER|REGION_FIT_BOUNDS;
     
-    return (region_attach_helper((WRegion*)dock, (WWindow*)dock, &fp,
-                                 (WRegionDoAttachFn*)dock_do_attach_final, 
-                                 NULL, data)
-            !=NULL);
+    return region_attach_helper((WRegion*)dock, (WWindow*)dock, &fp,
+                                (WRegionDoAttachFn*)dock_do_attach_final, 
+                                NULL, data);
 }
 
 
@@ -1417,7 +1410,7 @@ bool dock_attach(WDock *dock, WRegion *reg)
     data.type=REGION_ATTACH_REPARENT;
     data.u.reg=reg;
 
-    return dock_do_attach(dock, &data);
+    return (dock_do_attach(dock, &data)!=NULL);
 }
 
 
@@ -1428,7 +1421,7 @@ static bool dock_handle_drop(WDock *dock, int x, int y,
 }
 
 
-static bool dock_ph_handler(WDock *dock, int flags, WRegionAttachData *data)
+static WRegion *dock_ph_handler(WDock *dock, int flags, WRegionAttachData *data)
 {
     return dock_do_attach(dock, data);
 }
@@ -1444,9 +1437,9 @@ static WPHolder *dock_managed_get_pholder(WDock *dock, WRegion *mgd)
 
 static WPHolder *dock_prepare_manage(WDock *dock, const WClientWin *cwin,
                                      const WManageParams *param UNUSED,
-                                     int redir)
+                                     int priority)
 {
-    if(redir==MANAGE_REDIR_STRICT_YES)
+    if(!MANAGE_PRIORITY_OK(priority, MANAGE_PRIORITY_LOW))
         return NULL;
 
     return (WPHolder*)create_basicpholder((WRegion*)dock,
@@ -1590,7 +1583,7 @@ static bool clientwin_do_manage_hook(WClientWin *cwin, const WManageParams *para
     }
 
     return region_manage_clientwin((WRegion*)dock, cwin, param,
-                                   MANAGE_REDIR_PREFER_NO);
+                                   MANAGE_PRIORITY_NONE);
 }
 
 
@@ -1678,7 +1671,6 @@ static DynFunTab dock_dynfuntab[]={
     {region_size_hints, dock_size_hints},
     {(DynFun*)region_fitrep, (DynFun*)dock_fitrep},
     {(DynFun*)region_orientation, (DynFun*)dock_orientation},
-    {(DynFun*)region_may_destroy, (DynFun*)dock_may_destroy},
     {(DynFun*)region_handle_drop, (DynFun*)dock_handle_drop},
 
     {(DynFun*)region_managed_get_pholder,
