@@ -1,17 +1,15 @@
 /*
  * ion/mod_menu/mkmenu.c
  *
- * Copyright (c) Tuomo Valkonen 1999-2006. 
+ * Copyright (c) Tuomo Valkonen 1999-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #include <limits.h>
 #include <ioncore/common.h>
 #include <ioncore/pointer.h>
+#include <ioncore/stacking.h>
 #include <ioncore/grab.h>
 #include <libextl/extl.h>
 #include "menu.h"
@@ -49,11 +47,12 @@ WMenu *mod_menu_do_menu(WMPlex *mplex, ExtlFn handler, ExtlTab tab,
     fnp.refg.h=0;
     
     par.flags=(MPLEX_ATTACH_SWITCHTO|
-               MPLEX_ATTACH_MODAL|
+               MPLEX_ATTACH_LEVEL|
                MPLEX_ATTACH_UNNUMBERED|
                MPLEX_ATTACH_SIZEPOLICY);
     par.szplcy=SIZEPOLICY_FULL_BOUNDS;
-        
+    par.level=STACKING_LEVEL_MODAL1+1;
+    
     return (WMenu*)mplex_do_attach_new(mplex, &par,
                                        (WRegionCreateFn*)create_menu,
                                        (void*)&fnp); 
