@@ -1,12 +1,9 @@
 /*
  * ion/mod_query/complete.h
  *
- * Copyright (c) Tuomo Valkonen 1999-2006. 
+ * Copyright (c) Tuomo Valkonen 1999-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #ifndef ION_MOD_QUERY_COMPLETE_H
@@ -35,7 +32,7 @@ extern bool complproxy_set_completions(WComplProxy *proxy, ExtlTab compls);
 
 extern int edln_do_completions(Edln *edln, char **completions, int ncomp,
                                const char *beg, const char *end,
-                               bool setcommon);
+                               bool setcommon, bool nosort);
 extern void edln_set_completion(Edln *edln, const char *comp, 
                                 const char *beg, const char *end);
 
