@@ -1,12 +1,9 @@
 /*
  * ion/mod_query/query.c
  *
- * Copyright (c) Tuomo Valkonen 1999-2006. 
+ * Copyright (c) Tuomo Valkonen 1999-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 
@@ -14,16 +11,17 @@
 #include <ioncore/global.h>
 #include <ioncore/focus.h>
 #include <ioncore/frame.h>
+#include <ioncore/stacking.h>
 #include <libtu/objp.h>
 #include "wmessage.h"
 #include "fwarn.h"
 
 
-/*EXTL_DOC
+/*(internal) EXTL_DOC
  * Display an error message box in the multiplexer \var{mplex}.
  */
 EXTL_EXPORT
-WMessage *mod_query_warn(WMPlex *mplex, const char *p)
+WMessage *mod_query_do_warn(WMPlex *mplex, const char *p)
 {
     char *p2;
     WMessage *wmsg;
@@ -36,18 +34,19 @@ WMessage *mod_query_warn(WMPlex *mplex, const char *p)
     if(p2==NULL)
         return NULL;
     
-    wmsg=mod_query_message(mplex, p2);
+    wmsg=mod_query_do_message(mplex, p2);
     
     free(p2);
     
     return wmsg;
 }
 
-/*EXTL_DOC
+
+/*(internal) EXTL_DOC
  * Display a message in the \var{mplex}.
  */
 EXTL_EXPORT
-WMessage *mod_query_message(WMPlex *mplex, const char *p)
+WMessage *mod_query_do_message(WMPlex *mplex, const char *p)
 {
     WMPlexAttachParams par;
 
@@ -55,10 +54,11 @@ WMessage *mod_query_message(WMPlex *mplex, const char *p)
         return NULL;
     
     par.flags=(MPLEX_ATTACH_SWITCHTO|
-               MPLEX_ATTACH_MODAL|
+               MPLEX_ATTACH_LEVEL|
                MPLEX_ATTACH_UNNUMBERED|
                MPLEX_ATTACH_SIZEPOLICY);
     par.szplcy=SIZEPOLICY_FULL_BOUNDS;
+    par.level=STACKING_LEVEL_MODAL1+1;
 
     return (WMessage*)mplex_do_attach_new(mplex, &par,
                                           (WRegionCreateFn*)create_wmsg,
