@@ -1,12 +1,9 @@
 /*
  * ion/mod_query/history.h
  *
- * Copyright (c) Tuomo Valkonen 1999-2006. 
+ * Copyright (c) Tuomo Valkonen 1999-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #ifndef ION_MOD_QUERY_HISTORY_H
@@ -19,7 +16,9 @@ extern const char *mod_query_history_get(int n);
 extern bool mod_query_history_push(const char *s);
 extern void mod_query_history_push_(char *s);
 extern void mod_query_history_clear();
-extern int mod_query_history_search(const char *s, int from, bool bwd);
+extern int mod_query_history_search(const char *s, int from, bool bwd, 
+                                    bool exact);
+extern uint mod_query_history_complete(const char *s, char ***h_ret);
 extern ExtlTab mod_query_history_table();
 
 #endif /* ION_MOD_QUERY_HISTORY_H */
