@@ -1,12 +1,9 @@
 /*
  * ion/mod_query/input.h
  *
- * Copyright (c) Tuomo Valkonen 1999-2006. 
+ * Copyright (c) Tuomo Valkonen 1999-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #ifndef ION_MOD_QUERY_INPUT_H
@@ -39,4 +36,7 @@ DYNFUN void input_scrolldown(WInput *input);
 DYNFUN void input_calc_size(WInput *input, WRectangle *geom);
 DYNFUN const char *input_style(WInput *input);
 
+extern void mod_query_get_minimum_extents(GrBrush *brush, bool with_spacing, 
+                                          int *w, int *h);
+
 #endif /* ION_MOD_QUERY_INPUT_H */
