@@ -1,12 +1,9 @@
 /*
  * ion/mod_query/main.c
  *
- * Copyright (c) Tuomo Valkonen 1999-2006. 
+ * Copyright (c) Tuomo Valkonen 1999-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #include <libextl/readconfig.h>
@@ -55,7 +52,8 @@ WBindmap *mod_query_wedln_bindmap=NULL;
 
 ModQueryConfig mod_query_config={
     250,
-    TRUE
+    TRUE,
+    FALSE
 };
 
 
@@ -68,6 +66,8 @@ ModQueryConfig mod_query_config={
  *      (default: true). \\
  *  \var{autoshowcompl_delay} & (integer) auto-show-completions delay
  *      in milliseconds (default: 250). \\
+ *  \var{caseicompl} & (boolean) Turn some completions case-insensitive
+ *      (default: false). \\
  * \end{tabularx}
  */
 EXTL_EXPORT
@@ -76,6 +76,7 @@ void mod_query_set(ExtlTab tab)
     ModQueryConfig *c=&mod_query_config;
 
     extl_table_gets_b(tab, "autoshowcompl", &c->autoshowcompl);
+    extl_table_gets_b(tab, "caseicompl", &c->caseicompl);
     
     if(extl_table_gets_i(tab, "autoshowcompl_delay",
                          &c->autoshowcompl_delay)){
@@ -87,6 +88,7 @@ void mod_query_set(ExtlTab tab)
  * Get module configuration. For more information see
  * \fnref{mod_query.set}.
  */
+EXTL_SAFE
 EXTL_EXPORT
 ExtlTab mod_query_get()
 {
@@ -94,6 +96,7 @@ ExtlTab mod_query_get()
     ExtlTab tab=extl_create_table();
     
     extl_table_sets_b(tab, "autoshowcompl", c->autoshowcompl);
+    extl_table_sets_b(tab, "caseicompl", c->caseicompl);
     extl_table_sets_i(tab, "autoshowcompl_delay", c->autoshowcompl_delay);
     
     return tab;
