@@ -1,12 +1,9 @@
 /*
  * ion/mod_query/main.h
  *
- * Copyright (c) Tuomo Valkonen 1999-2006. 
+ * Copyright (c) Tuomo Valkonen 1999-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #ifndef ION_MOD_QUERY_MAIN_H
@@ -20,6 +17,7 @@ INTRSTRUCT(ModQueryConfig);
 DECLSTRUCT(ModQueryConfig){
     int autoshowcompl_delay;
     bool autoshowcompl;
+    bool caseicompl;
 };
 
 
