@@ -1,15 +1,13 @@
 /*
  * ion/mod_statusbar/statusbar.c
  *
- * Copyright (c) Tuomo Valkonen 1999-2006. 
+ * Copyright (c) Tuomo Valkonen 1999-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #include <string.h>
+#include <limits.h>
 
 #include <libtu/objp.h>
 #include <libtu/minmax.h>
@@ -23,6 +21,7 @@
 #include <ioncore/event.h>
 #include <ioncore/resize.h>
 #include <ioncore/gr.h>
+#include <ioncore/gr-util.h>
 #include <ioncore/names.h>
 #include <ioncore/strings.h>
 #include <ioncore/basicpholder.h>
@@ -42,7 +41,6 @@ static void statusbar_rearrange(WStatusBar *sb, bool rs);
 static void do_calc_systray_w(WStatusBar *p, WSBElem *el);
 static void statusbar_calc_systray_w(WStatusBar *p);
 
-
 static WStatusBar *statusbars=NULL;
 
 
@@ -113,15 +111,16 @@ void statusbar_deinit(WStatusBar *p)
 
 /*{{{ Content stuff */
 
+
 static void init_sbelem(WSBElem *el)
 {
     el->type=WSBELEM_NONE;
-    el->meter=NULL;
     el->text_w=0;
     el->text=NULL;
     el->max_w=0;
     el->tmpl=NULL;
-    el->attr=NULL;
+    el->meter=STRINGID_NONE;
+    el->attr=STRINGID_NONE;
     el->stretch=0;
     el->align=WSBELEM_ALIGN_CENTER;
     el->zeropad=0;
@@ -130,6 +129,20 @@ static void init_sbelem(WSBElem *el)
 }
 
 
+static bool gets_stringstore(ExtlTab t, const char *str, StringId *id)
+{
+    char *s;
+    
+    if(extl_table_gets_s(t, str, &s)){
+        *id=stringstore_alloc(s);
+        free(s);
+        return (*id!=STRINGID_NONE);
+    }
+    
+    return FALSE;
+}
+
+    
 static WSBElem *get_sbelems(ExtlTab t, int *nret, int *filleridxret)
 {
     int i, n=extl_table_get_n(t);
@@ -157,15 +170,20 @@ static WSBElem *get_sbelems(ExtlTab t, int *nret, int *filleridxret)
                 if(el[i].type==WSBELEM_TEXT || el[i].type==WSBELEM_STRETCH){
                     extl_table_gets_s(tt, "text", &(el[i].text));
                 }else if(el[i].type==WSBELEM_METER){
-                    extl_table_gets_s(tt, "meter", &(el[i].meter));
+                    gets_stringstore(tt, "meter", &(el[i].meter));
                     extl_table_gets_s(tt, "tmpl", &(el[i].tmpl));
                     extl_table_gets_i(tt, "align", &(el[i].align));
                     extl_table_gets_i(tt, "zeropad", &(el[i].zeropad));
                     el[i].zeropad=maxof(el[i].zeropad, 0);
                 }else if(el[i].type==WSBELEM_SYSTRAY){
-                    extl_table_gets_s(tt, "meter", &(el[i].meter));
+                    const char *tmp;
+                    
+                    gets_stringstore(tt, "meter", &(el[i].meter));
                     extl_table_gets_i(tt, "align", &(el[i].align));
-                    if(el[i].meter==NULL || strcmp(el[i].meter, "systray")==0)
+                    
+                    tmp=stringstore_get(el[i].meter);
+                    
+                    if(tmp==NULL || strcmp(tmp, "systray")==0)
                         systrayidx=i;
                 }else if(el[i].type==WSBELEM_FILLER){
                     *filleridxret=i;
@@ -198,12 +216,12 @@ static void free_sbelems(WSBElem *el, int n)
     for(i=0; i<n; i++){
         if(el[i].text!=NULL)
             free(el[i].text);
-        if(el[i].meter!=NULL)
-            free(el[i].meter);
         if(el[i].tmpl!=NULL)
             free(el[i].tmpl);
-        if(el[i].attr!=NULL)
-            free(el[i].attr);
+        if(el[i].meter!=STRINGID_NONE)
+            stringstore_free(el[i].meter);
+        if(el[i].attr!=STRINGID_NONE)
+            stringstore_free(el[i].attr);
         if(el[i].traywins!=NULL)
             ptrlist_clear(&el[i].traywins);
     }
@@ -240,16 +258,17 @@ static void statusbar_free_elems(WStatusBar *sb)
 
 static void statusbar_resize(WStatusBar *p)
 {
-    int rqflags=REGION_RQGEOM_WEAK_X|REGION_RQGEOM_WEAK_Y;
-    WRectangle g;
+    WRQGeomParams rq=RQGEOMPARAMS_INIT;
     
-    g.w=p->natural_w;
-    g.h=p->natural_h;
-    g.x=REGION_GEOM(p).x;
-    g.y=REGION_GEOM(p).y;
+    rq.flags=REGION_RQGEOM_WEAK_X|REGION_RQGEOM_WEAK_Y;
+    
+    rq.geom.w=p->natural_w;
+    rq.geom.h=p->natural_h;
+    rq.geom.x=REGION_GEOM(p).x;
+    rq.geom.y=REGION_GEOM(p).y;
 
-    if(g.w!=REGION_GEOM(p).w || g.h!=REGION_GEOM(p).h)
-    region_rqgeom((WRegion*)p, rqflags, &g, NULL);
+    if(rectangle_compare(&rq.geom, &REGION_GEOM(p))!=RECTANGLE_SAME)
+        region_rqgeom((WRegion*)p, &rq, NULL);
 }
 
 
@@ -331,7 +350,7 @@ void statusbar_size_hints(WStatusBar *p, WSizeHints *h)
     h->min_height=p->natural_h;
     
     h->max_set=TRUE;
-    h->max_width=p->natural_w;
+    h->max_width=INT_MAX;/*p->natural_w;*/
     h->max_height=p->natural_h;
 }
 
@@ -353,17 +372,22 @@ static WSBElem *statusbar_associate_systray(WStatusBar *sb, WRegion *reg)
         extl_table_gets_s(cwin->proptab, "statusbar", &name);
     
     for(i=0; i<sb->nelems; i++){
+        const char *meter;
+        
         if(sb->elems[i].type!=WSBELEM_SYSTRAY)
             continue;
-        if(sb->elems[i].meter==NULL){
+        
+        meter=stringstore_get(sb->elems[i].meter);
+        
+        if(meter==NULL){
             fbel=&sb->elems[i];
             continue;
         }
-        if(name!=NULL && strcmp(sb->elems[i].meter, name)==0){
+        if(name!=NULL && strcmp(meter, name)==0){
             el=&sb->elems[i];
             break;
         }
-        if(strcmp(sb->elems[i].meter, "systray")==0)
+        if(strcmp(meter, "systray")==0)
             fbel=&sb->elems[i];
     }
     
@@ -500,7 +524,7 @@ static WRegion *statusbar_do_attach_final(WStatusBar *sb,
 }
 
 
-static bool statusbar_do_attach(WStatusBar *sb, WRegionAttachData *data)
+static WRegion *statusbar_do_attach(WStatusBar *sb, WRegionAttachData *data)
 {
     WFitParams fp;
     
@@ -510,15 +534,14 @@ static bool statusbar_do_attach(WStatusBar *sb, WRegionAttachData *data)
     fp.g.w=CF_STATUSBAR_SYSTRAY_HEIGHT;
     fp.mode=REGION_FIT_WHATEVER|REGION_FIT_BOUNDS;
     
-    return (region_attach_helper((WRegion*)sb, (WWindow*)sb, &fp,
-                                 (WRegionDoAttachFn*)statusbar_do_attach_final, 
-                                 NULL, data)
-            !=NULL);
+    return region_attach_helper((WRegion*)sb, (WWindow*)sb, &fp,
+                                (WRegionDoAttachFn*)statusbar_do_attach_final, 
+                                NULL, data);
 }
 
 
-static bool statusbar_attach_ph(WStatusBar *sb, int flags,
-                                WRegionAttachData *data)
+static WRegion *statusbar_attach_ph(WStatusBar *sb, int flags,
+                                    WRegionAttachData *data)
 {
     return statusbar_do_attach(sb, data);
 }
@@ -527,9 +550,9 @@ static bool statusbar_attach_ph(WStatusBar *sb, int flags,
 static WPHolder *statusbar_prepare_manage(WStatusBar *sb, 
                                           const WClientWin *cwin,
                                           const WManageParams *param,
-                                          int redir)
+                                          int priority)
 {
-    if(redir==MANAGE_REDIR_STRICT_YES)
+    if(!MANAGE_PRIORITY_OK(priority, MANAGE_PRIORITY_LOW))
         return NULL;
     
     return (WPHolder*)create_basicpholder((WRegion*)sb, 
@@ -555,20 +578,20 @@ static void statusbar_managed_remove(WStatusBar *sb, WRegion *reg)
 }
 
 
-static void statusbar_managed_rqgeom(WStatusBar *sb, WRegion *reg, int flags,
-                                     const WRectangle *geom, 
+static void statusbar_managed_rqgeom(WStatusBar *sb, WRegion *reg, 
+                                     const WRQGeomParams *rq,
                                      WRectangle *geomret)
 {
     WRectangle g;
     
     g.x=REGION_GEOM(reg).x;
     g.y=REGION_GEOM(reg).y;
-    g.w=geom->w;
-    g.h=geom->h;
+    g.w=rq->geom.w;
+    g.h=rq->geom.h;
 
     systray_adjust_size(reg, &g);
 
-    if(flags&REGION_RQGEOM_TRYONLY){
+    if(rq->flags&REGION_RQGEOM_TRYONLY){
         if(geomret!=NULL)
             *geomret=g;
         return;
@@ -638,7 +661,7 @@ WPHolder *statusbar_prepare_manage_transient(WStatusBar *sb,
     
     if(mgr!=NULL)
         return region_prepare_manage(mgr, cwin, param, 
-                                     MANAGE_REDIR_PREFER_NO);
+                                     MANAGE_PRIORITY_NONE);
     else
         return NULL;
 }
@@ -721,7 +744,7 @@ ExtlTab statusbar_get_template_table(WStatusBar *sb)
         
         extl_table_sets_i(tt, "type", sb->elems[i].type);
         extl_table_sets_s(tt, "text", sb->elems[i].text);
-        extl_table_sets_s(tt, "meter", sb->elems[i].meter);
+        extl_table_sets_s(tt, "meter", stringstore_get(sb->elems[i].meter));
         extl_table_sets_s(tt, "tmpl", sb->elems[i].tmpl);
         extl_table_sets_i(tt, "align", sb->elems[i].align);
         extl_table_sets_i(tt, "zeropad", sb->elems[i].zeropad);
@@ -844,6 +867,8 @@ void statusbar_update(WStatusBar *sb, ExtlTab t)
         return;
     
     for(i=0; i<sb->nelems; i++){
+        const char *meter;
+        
         el=&(sb->elems[i]);
         
         if(el->type!=WSBELEM_METER)
@@ -854,16 +879,18 @@ void statusbar_update(WStatusBar *sb, ExtlTab t)
             el->text=NULL;
         }
 
-        if(el->attr!=NULL){
-            free(el->attr);
-            el->attr=NULL;
+        if(el->attr!=GRATTR_NONE){
+            stringstore_free(el->attr);
+            el->attr=GRATTR_NONE;
         }
         
-        if(el->meter!=NULL){
+        meter=stringstore_get(el->meter);
+        
+        if(meter!=NULL){
             const char *str;
             char *attrnm;
             
-            extl_table_gets_s(t, el->meter, &(el->text));
+            extl_table_gets_s(t, meter, &(el->text));
             
             if(el->text==NULL){
                 str=STATUSBAR_NX_STR;
@@ -900,9 +927,13 @@ void statusbar_update(WStatusBar *sb, ExtlTab t)
                 grow=TRUE;
             }
             
-            attrnm=scat(el->meter, "_hint");
+            attrnm=scat(meter, "_hint");
             if(attrnm!=NULL){
-                extl_table_gets_s(t, attrnm, &(el->attr));
+                char *s;
+                if(extl_table_gets_s(t, attrnm, &s)){
+                    el->attr=stringstore_alloc(s);
+                    free(s);
+                }
                 free(attrnm);
             }
         }
@@ -1004,7 +1035,8 @@ bool statusbar_set_systray(WStatusBar *sb, int sp)
 
 /*EXTL_DOC
  * Enable or disable use of \var{sb} as systray.
- * The parameter \var{how} can be one of (set/unset/toggle). 
+ * The parameter \var{how} can be one of 
+ * \codestr{set}, \codestr{unset}, or \codestr{toggle}.
  * Resulting state is returned.
  */
 EXTL_EXPORT_AS(WStatusBar, set_systray)
@@ -1017,7 +1049,7 @@ bool statusbar_set_systray_extl(WStatusBar *sb, const char *how)
 /*EXTL_DOC
  * Is \var{sb} used as a systray?
  */
-EXTL_EXPORT_MEMBER
+EXTL_EXPORT_AS(WStatusBar, is_systray)
 bool statusbar_is_systray_extl(WStatusBar *sb)
 {
     return sb->systray_enabled;
