@@ -1,12 +1,9 @@
 /*
  * ion/pwm/pwm.c
  *
- * Copyright (c) Tuomo Valkonen 1999-2006. 
+ * Copyright (c) Tuomo Valkonen 1999-2007. 
  *
- * Ion is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation; either version 2.1 of the License, or
- * (at your option) any later version.
+ * See the included file LICENSE for details.
  */
 
 #include <stdlib.h>
@@ -48,16 +45,8 @@ static OptParserOpt pwm_opts[]={
      DUMMY_TR("Add directory to search path")},
 
     {OPT_ID('o'), "oneroot",  0, NULL,
-     DUMMY_TR("Manage default root window/non-Xinerama screen only")},
+     DUMMY_TR("Manage default screen only")},
 
-#if defined(CF_XINERAMA) || defined(CF_SUN_XINERAMA)
-    {OPT_ID('x'), "xinerama", OPT_ARG, "1|0",
-     DUMMY_TR("Use Xinerama screen information (default: 0/no)")},
-#else
-    {OPT_ID('x'), "xinerama", OPT_ARG, "?",
-     DUMMY_TR("Ignored: not compiled with Xinerama support")},
-#endif
-    
     {OPT_ID('s'), "session",  OPT_ARG, "session_name", 
      DUMMY_TR("Name of session (affects savefiles)")},
     
@@ -84,7 +73,7 @@ static OptParserOpt pwm_opts[]={
 static void help()
 {
     int i;
-    printf(TR("Usage: %s [options]\n\n"), prog_execname());
+    printf(TR("Usage: %s [options]\n\n"), libtu_progname());
     for(i=0; pwm_opts[i].descr!=NULL; i++)
         pwm_opts[i].descr=TR(pwm_opts[i].descr);
     optparser_printhelp(OPTP_MIDLONG, pwm_opts);
@@ -97,7 +86,7 @@ int main(int argc, char*argv[])
     const char *cfgfile="cfg_pwm";
     const char *display=NULL;
     char *cmd=NULL;
-    int stflags=IONCORE_STARTUP_NOXINERAMA;
+    int stflags=0;
     int opt;
     ErrorLog el;
     FILE *ef=NULL;
@@ -139,17 +128,6 @@ int main(int argc, char*argv[])
         case OPT_ID('o'):
             stflags|=IONCORE_STARTUP_ONEROOT;
             break;
-        case OPT_ID('x'):
-            {
-                const char *p=optparser_get_arg();
-                if(strcmp(p, "1")==0)
-                    stflags&=~IONCORE_STARTUP_NOXINERAMA;
-                else if(strcmp(p, "0")==0)
-                    stflags|=IONCORE_STARTUP_NOXINERAMA;
-                else
-                    warn(TR("Invalid parameter to -xinerama."));
-            }
-            break;
         case OPT_ID('s'):
             extl_set_sessiondir(optparser_get_arg());
             break;
@@ -207,7 +185,7 @@ fail:
             fclose(ef);
             pid=fork();
             if(pid==0){
-                ioncore_setup_environ(DefaultScreen(ioncore_g.dpy));
+                ioncore_setup_display(DefaultScreen(ioncore_g.dpy));
                 if(!may_continue)
                     XCloseDisplay(ioncore_g.dpy);
                 else
